"""Generate a diffusion stencil

Supports isotropic diffusion (FE,FD), anisotropic diffusion (FE, FD), and
rotated anisotropic diffusion (FD).

The stencils include redundancy to maintain readability for simple cases (e.g.
isotropic diffusion).

"""

import numpy as np

__docformat__ = "restructuredtext en"

__all__ = ['diffusion_stencil_2d']


def diffusion_stencil_2d(epsilon=1.0, theta=0.0, type='FE'):
    """
    Rotated Anisotropic diffusion in 2d of the form:

    -div Q A Q^T grad u

    Q = [cos(theta) -sin(theta)]
        [sin(theta)  cos(theta)]

    A = [1          0        ]
        [0          eps      ]

    Parameters
    ----------
    epsilon  : float, optional
        Anisotropic diffusion coefficient: -div A grad u,
        where A = [1 0; 0 epsilon].  The default is isotropic, epsilon=1.0
    theta : float, optional
        Rotation angle `theta` in radians defines -div Q A Q^T grad,
        where Q = [cos(`theta`) -sin(`theta`); sin(`theta`) cos(`theta`)].
    type : {'FE','FD'}
        Specifies the discretization as Q1 finite element (FE) or 2nd order
        finite difference (FD)
        The default is `theta` = 0.0

    Returns
    -------
    stencil : numpy array
        A 3x3 diffusion stencil

    See Also
    --------
    stencil_grid, poisson

    Notes
    -----
    Not all combinations are supported.

    Examples
    --------
    >>> import scipy
    >>> from pyamg.gallery.diffusion import diffusion_stencil_2d
    >>> sten = diffusion_stencil_2d(epsilon=0.0001,theta=scipy.pi/6,type='FD')
    >>> print sten
    [[-0.2164847 -0.750025   0.2164847]
     [-0.250075   2.0002    -0.250075 ]
     [ 0.2164847 -0.750025  -0.2164847]]

    """
    eps = float(epsilon)  # for brevity
    theta = float(theta)

    C = np.cos(theta)
    S = np.sin(theta)
    CS = C*S
    CC = C**2
    SS = S**2

    if(type == 'FE'):
        """FE approximation to::

            - (eps c^2 +     s^2) u_xx +
            -2(eps - 1) c s       u_xy +
            - (    c^2 + eps s^2) u_yy

            [ -c^2*eps-s^2+3*c*s*(eps-1)-c^2-s^2*eps,
              2*c^2*eps+2*s^2-4*c^2-4*s^2*eps,
              -c^2*eps-s^2-3*c*s*(eps-1)-c^2-s^2*eps]

            [-4*c^2*eps-4*s^2+2*c^2+2*s^2*eps,
             8*c^2*eps+8*s^2+8*c^2+8*s^2*eps,
             -4*c^2*eps-4*s^2+2*c^2+2*s^2*eps]

            [-c^2*eps-s^2-3*c*s*(eps-1)-c^2-s^2*eps,
             2*c^2*eps+2*s^2-4*c^2-4*s^2*eps,
             -c^2*eps-s^2+3*c*s*(eps-1)-c^2-s^2*eps]

            c = cos(theta)
            s = sin(theta)
        """

        a = (-1*eps - 1)*CC + (-1*eps - 1)*SS + (3*eps - 3)*CS
        b = (2*eps - 4)*CC + (-4*eps + 2)*SS
        c = (-1*eps - 1)*CC + (-1*eps - 1)*SS + (-3*eps + 3)*CS
        d = (-4*eps + 2)*CC + (2*eps - 4)*SS
        e = (8*eps + 8)*CC + (8*eps + 8)*SS

        stencil = np.array([[a, b, c],
                            [d, e, d],
                            [c, b, a]]) / 6.0

    elif type == 'FD':
        """FD approximation to:

        - (eps c^2 +     s^2) u_xx +
        -2(eps - 1) c s       u_xy +
        - (    c^2 + eps s^2) u_yy

          c = cos(theta)
          s = sin(theta)

        A = [ 1/2(eps - 1) c s    -(c^2 + eps s^2)    -1/2(eps - 1) c s  ]
            [                                                            ]
            [ -(eps c^2 + s^2)       2 (eps + 1)    -(eps c^2 + s^2)     ]
            [                                                            ]
            [  -1/2(eps - 1) c s    -(c^2 + eps s^2)  1/2(eps - 1) c s   ]
        """

        a = 0.5*(eps - 1)*CS
        b = -(eps*SS + CC)
        c = -a
        d = -(eps*CC + SS)
        e = 2.0*(eps + 1)

        stencil = np.array([[a, b, c],
                            [d, e, d],
                            [c, b, a]])

    return stencil


def _symbolic_rotation_helper():
    """
    Simple SymPy script to generate the 3D rotation matrix and products for
    diffusion_stencil_3d.

    """

    from sympy import symbols, Matrix

    cpsi, spsi = symbols('cpsi, spsi')
    cth, sth = symbols('cth, sth')
    cphi, sphi = symbols('cphi, sphi')
    Rpsi = Matrix([[cpsi, spsi, 0], [-spsi, cpsi, 0], [0, 0, 1]])
    Rth = Matrix([[1, 0, 0], [0, cth, sth], [0, -sth, cth]])
    Rphi = Matrix([[cphi, sphi, 0], [-sphi, cphi, 0], [0, 0, 1]])

    Q = Rpsi * Rth * Rphi

    epsy, epsz = symbols('epsy, epsz')
    A = Matrix([[1, 0, 0], [0, epsy, 0], [0, 0, epsz]])

    D = Q * A * Q.T

    for i in range(3):
        for j in range(3):
            print 'D[%d, %d] = %s' % (i, j, D[i, j])


def _symbolic_product_helper():
    """
    Simple SymPy script to generate the 3D products for diffusion_stencil_3d.

    """

    from sympy import symbols, Matrix

    D11, D12, D13, D21, D22, D23, D31, D32, D33 =\
        symbols('D11, D12, D13, D21, D22, D23, D31, D32, D33')

    D = Matrix([[D11, D12, D13], [D21, D22, D23], [D31, D32, D33]])
    grad = Matrix([['dx', 'dy', 'dz']]).T
    div = grad.T

    a = div * D * grad

    print a[0]


def diffusion_stencil_3d(epsilony=1.0, epsilonz=1.0, theta=0.0, phi=0.0,
                         psi=0.0, type='FD'):
    """
    Rotated Anisotropic diffusion in 3d of the form:

    -div Q A Q^T grad u

    Q = Rpsi Rtheta Rphi

    Rpsi = [ c   s   0 ]
           [-s   c   0 ]
           [ 0   0   1 ]
           c = cos(psi)
           s = sin(psi)

    Rtheta = [ 1   0   0 ]
             [ 0   c   s ]
             [ 0  -s   c ]
           c = cos(theta)
           s = sin(theta)

    Rphi = [ c   s   0 ]
           [-s   c   0 ]
           [ 0   0   1 ]
           c = cos(phi)
           s = sin(phi)

    Here Euler Angles are used:
    http://en.wikipedia.org/wiki/Euler_angles

    This results in

    Q = [   cphi*cpsi - cth*sphi*spsi, cpsi*sphi + cphi*cth*spsi, spsi*sth]
        [ - cphi*spsi - cpsi*cth*sphi, cphi*cpsi*cth - sphi*spsi, cpsi*sth]
        [                    sphi*sth,                 -cphi*sth,      cth]

    A = [1          0            ]
        [0          epsy         ]
        [0          0        epsz]

    D = Q A Q^T

    Parameters
    ----------
    epsilony  : float, optional
        Anisotropic diffusion coefficient in the y-direction
        where A = [1 0 0; 0 epsilon_y 0; 0 0 epsilon_z].  The default is
        isotropic, epsilon=1.0
    epsilonz  : float, optional
        Anisotropic diffusion coefficient in the z-direction
        where A = [1 0 0; 0 epsilon_y 0; 0 0 epsilon_z].  The default is
        isotropic, epsilon=1.0
    theta : float, optional
        Euler rotation angle `theta` in radians. The default is 0.0.
    phi : float, optional
        Euler rotation angle `phi` in radians. The default is 0.0.
    psi : float, optional
        Euler rotation angle `psi` in radians. The default is 0.0.
    type : {'FE','FD'}
        Specifies the discretization as Q1 finite element (FE) or 2nd order
        finite difference (FD)

    Returns
    -------
    stencil : numpy array
        A 3x3 diffusion stencil

    See Also
    --------
    stencil_grid, poisson, _symbolic_rotation_helper, _symbolic_product_helper

    Notes
    -----
    Not all combinations are supported.

    Examples
    --------
    >>> import scipy
    >>> from pyamg.gallery.diffusion import diffusion_stencil_2d
    >>> sten = diffusion_stencil_2d(epsilon=0.0001,theta=scipy.pi/6,type='FD')
    >>> print sten
    [[-0.2164847 -0.750025   0.2164847]
     [-0.250075   2.0002    -0.250075 ]
     [ 0.2164847 -0.750025  -0.2164847]]

    """

    epsy = float(epsilony)  # for brevity
    epsz = float(epsilonz)  # for brevity
    theta = float(theta)
    phi = float(phi)
    psi = float(psi)

    D = np.zeros((3, 3))
    cphi = np.cos(phi)
    sphi = np.sin(phi)
    cth = np.cos(theta)
    sth = np.sin(theta)
    cpsi = np.cos(psi)
    spsi = np.sin(psi)

    # from _symbolic_rotation_helper
    D[0, 0] = epsy*(cphi*cth*spsi + cpsi*sphi)**2 + epsz*spsi**2*sth**2 +\
        (cphi*cpsi - cth*sphi*spsi)**2
    D[0, 1] = cpsi*epsz*spsi*sth**2 +\
        epsy*(cphi*cpsi*cth - sphi*spsi)*(cphi*cth*spsi + cpsi*sphi) +\
        (cphi*cpsi - cth*sphi*spsi)*(-cphi*spsi - cpsi*cth*sphi)
    D[0, 2] = -cphi*epsy*sth*(cphi*cth*spsi + cpsi*sphi) +\
        cth*epsz*spsi*sth + sphi*sth*(cphi*cpsi - cth*sphi*spsi)
    D[1, 0] = cpsi*epsz*spsi*sth**2 +\
        epsy*(cphi*cpsi*cth - sphi*spsi)*(cphi*cth*spsi + cpsi*sphi) +\
        (cphi*cpsi - cth*sphi*spsi)*(-cphi*spsi - cpsi*cth*sphi)
    D[1, 1] = cpsi**2*epsz*sth**2 + epsy*(cphi*cpsi*cth - sphi*spsi)**2 +\
        (-cphi*spsi - cpsi*cth*sphi)**2
    D[1, 2] = -cphi*epsy*sth*(cphi*cpsi*cth - sphi*spsi) +\
        cpsi*cth*epsz*sth + sphi*sth*(-cphi*spsi - cpsi*cth*sphi)
    D[2, 0] = -cphi*epsy*sth*(cphi*cth*spsi + cpsi*sphi) + cth*epsz*spsi*sth +\
        sphi*sth*(cphi*cpsi - cth*sphi*spsi)
    D[2, 1] = -cphi*epsy*sth*(cphi*cpsi*cth - sphi*spsi) + cpsi*cth*epsz*sth +\
        sphi*sth*(-cphi*spsi - cpsi*cth*sphi)
    D[2, 2] = cphi**2*epsy*sth**2 + cth**2*epsz + sphi**2*sth**2

    stencil = np.zeros((3, 3, 3))

    if type == 'FD':
        # from _symbolic_product_helper
        # dx*(D11*dx + D21*dy + D31*dz) +
        # dy*(D12*dx + D22*dy + D32*dz) +
        # dz*(D13*dx + D23*dy + D33*dz)
        #
        # D00*dxx +
        # (D10+D01)*dxy +
        # (D20+D02)*dxz +
        # D11*dyy +
        # (D21+D12)*dyz +
        # D22*dzz

        i, j, k = (1, 1, 1)

        # dxx
        stencil[[i-1, i, i+1], j, k] += np.array([-1, 2, -1]) * D[0, 0]

        # dyy
        stencil[i, [j-1, j, j+1], k] += np.array([-1, 2, -1]) * D[1, 1]

        # dzz
        stencil[i, j, [k-1, k, k+1]] += np.array([-1, 2, -1]) * D[2, 2]

        L = np.array([-1, -1, 1, 1])
        M = np.array([-1, 1, -1, 1])
        # dxy
        stencil[i + L, j + M, k] \
            += 0.25 * np.array([1, -1, -1, 1]) * (D[1, 0] + D[0, 1])

        # dxz
        stencil[i + L, j, k + M] \
            += 0.25 * np.array([1, -1, -1, 1]) * (D[2, 0] + D[0, 2])

        # dyz
        stencil[i, j + L, k + M] \
            += 0.25 * np.array([1, -1, -1, 1]) * (D[2, 1] + D[1, 2])

        return stencil

    if type == 'FE':
        raise NotImplementedError('FE not implemented yet')



from pyamg.testing import *

import numpy
from numpy import ravel, ones, concatenate, cumsum, zeros
from scipy import rand
from scipy.sparse import csr_matrix, lil_matrix, coo_matrix

from pyamg.gallery import poisson, load_example
from pyamg.strength import classical_strength_of_connection

from pyamg.classical import split
from pyamg.classical.classical import ruge_stuben_solver
from pyamg.classical.interpolate import direct_interpolation

class TestRugeStubenFunctions(TestCase):
    def setUp(self):
        self.cases = []

        # random matrices
        numpy.random.seed(0)
        for N in [2,3,5]:
            self.cases.append( csr_matrix(rand(N,N)) )

        # Poisson problems in 1D and 2D
        for N in [2,3,5,7,10,11,19]:
            self.cases.append( poisson( (N,), format='csr') )
        for N in [2,3,5,7,10,11]:
            self.cases.append( poisson( (N,N), format='csr') )

        for name in ['knot','airfoil','bar']:
            ex = load_example(name)
            self.cases.append( ex['A'].tocsr() )

    def test_RS_splitting(self):
        for A in self.cases:
            S = classical_strength_of_connection(A, 0.0)

            splitting = split.RS( S )

            assert( splitting.min() >= 0 )     #could be all 1s
            assert_equal( splitting.max(), 1 ) 

            S.data[:] = 1

            # check that all F-nodes are strongly connected to a C-node
            assert( (splitting + S*splitting).min() > 0 )

            ### THIS IS NOT STRICTLY ENFORCED!
            ## check that all strong connections S[i,j] satisfy either:
            ## (0) i is a C-node
            ## (1) j is a C-node
            ## (2) k is a C-node and both i and j are are strongly connected to k
            #
            #X = S.tocoo()

            ## remove C->F edges (i.e. S[i,j] where (0) holds )
            #mask = splitting[X.row] == 0
            #X.row  = X.row[mask]
            #X.col  = X.col[mask]
            #X.data = X.data[mask]

            ## remove F->C edges (i.e. S[i,j] where (1) holds )
            #mask = splitting[X.col] == 0 
            #X.row  = X.row[mask]
            #X.col  = X.col[mask] 
            #X.data = X.data[mask]

            ## X now consists of strong F->F edges only
            #
            ## (S * S.T)[i,j] is the # of C nodes on which both i and j 
            ## strongly depend (i.e. the number of k's where (2) holds)
            #Y = (S*S.T) - X
            #assert( Y.nnz == 0 or Y.data.min() > 0 )

    def test_cljp_splitting(self):
        for A in self.cases:
            S = classical_strength_of_connection(A, 0.0)

            splitting = split.CLJP( S )

            assert( splitting.min() >= 0 )     #could be all 1s
            assert_equal( splitting.max(), 1 ) 

            S.data[:] = 1

            # check that all F-nodes are strongly connected to a C-node
            assert( (splitting + S*splitting).min() > 0 )

    def test_cljpc_splitting(self):
        for A in self.cases:
            S = classical_strength_of_connection(A, 0.0)

            splitting = split.CLJPc( S )

            assert( splitting.min() >= 0 )     #could be all 1s
            assert_equal( splitting.max(), 1 ) 

            S.data[:] = 1

            # check that all F-nodes are strongly connected to a C-node
            assert( (splitting + S*splitting).min() > 0 )
   
    def test_direct_interpolation(self):
        for A in self.cases:

            S = classical_strength_of_connection(A, 0.0)
            splitting = split.RS( S )

            result   = direct_interpolation(A,S,splitting)                
            expected = reference_direct_interpolation( A, S, splitting )

            assert_almost_equal( result.todense(), expected.todense() )

            

class TestSolverPerformance(TestCase):
    def test_poisson(self):
        cases = []
        
        cases.append( (500,) )
        cases.append( (250,250) )
        cases.append( (25,25,25) )

        for case in cases:
            A = poisson( case, format='csr' )

            numpy.random.seed(0) #make tests repeatable

            x = rand(A.shape[0])
            b = A*rand(A.shape[0]) #zeros_like(x)

            ml = ruge_stuben_solver(A, max_coarse=50)

            residuals = []
            x_sol = ml.solve(b, x0=x, maxiter=20, tol=1e-12, residuals=residuals)

            avg_convergence_ratio = (residuals[-1]/residuals[0])**(1.0/len(residuals))
            
            assert(avg_convergence_ratio < 0.20)


    def test_matrix_formats(self):

        # Do dense, csr, bsr and csc versions of A all yield the same solver
        A = poisson( (7,7), format='csr')
        cases = [ A.tobsr(blocksize=(1,1)) ]
        cases.append(A.tocsc())
        cases.append(A.todense())
        
        rs_old = ruge_stuben_solver(A,max_coarse=10)
        for AA in cases:
            rs_new = ruge_stuben_solver(AA,max_coarse=10)
            assert( abs( ravel( rs_old.levels[-1].A.todense() -
                         rs_new.levels[-1].A.todense() )).max() < 0.01 )
            rs_old = rs_new


################################################
##   reference implementations for unittests  ##
################################################


def reference_direct_interpolation(A,S,splitting):

    ##
    # Interpolation weights are computed based on entries in A, but subject to
    # the sparsity pattern of C.  So, we copy the entries of A into the
    # sparsity pattern of C.
    S = S.copy()
    S.data[:] = 1.0
    S = S.multiply(A)

    A = coo_matrix(A)
    S = coo_matrix(S)  

    # remove diagonals
    mask = S.row != S.col
    S.row  = S.row[mask]
    S.col  = S.col[mask]
    S.data = S.data[mask]

    #strong C points
    c_mask = splitting[S.col] == 1
    C_s = coo_matrix( (S.data[c_mask],(S.row[c_mask],S.col[c_mask])), shape=S.shape)

    #strong F points
    f_mask = ~c_mask
    F_s = coo_matrix( (S.data[f_mask],(S.row[f_mask],S.col[f_mask])), shape=S.shape)

    # split A in to + and -
    mask = (A.data > 0) & (A.row != A.col)
    A_pos = coo_matrix( (A.data[mask],(A.row[mask],A.col[mask])), shape=A.shape)
    mask = (A.data < 0) & (A.row != A.col)
    A_neg = coo_matrix( (A.data[mask],(A.row[mask],A.col[mask])), shape=A.shape)

    # split C_S in to + and -
    mask = C_s.data > 0
    C_s_pos = coo_matrix( (C_s.data[mask],(C_s.row[mask],C_s.col[mask])), shape=A.shape)
    mask = ~mask
    C_s_neg = coo_matrix( (C_s.data[mask],(C_s.row[mask],C_s.col[mask])), shape=A.shape)

    sum_strong_pos = ravel(C_s_pos.sum(axis=1))
    sum_strong_neg = ravel(C_s_neg.sum(axis=1))

    sum_all_pos = ravel(A_pos.sum(axis=1))
    sum_all_neg = ravel(A_neg.sum(axis=1))

    diag = A.diagonal()

    mask = (sum_strong_neg != 0.0)
    alpha = numpy.zeros_like(sum_all_neg)
    alpha[mask] = sum_all_neg[mask] / sum_strong_neg[mask]
    
    mask = (sum_strong_pos != 0.0)
    beta = numpy.zeros_like(sum_all_pos)
    beta[mask]  = sum_all_pos[mask] / sum_strong_pos[mask]

    mask = sum_strong_pos == 0
    diag[mask] += sum_all_pos[mask]
    beta[mask] = 0

    C_s_neg.data *= -alpha[C_s_neg.row]/diag[C_s_neg.row]
    C_s_pos.data *= -beta[C_s_pos.row]/diag[C_s_pos.row]
    
    C_rows = splitting.nonzero()[0] 
    C_inject = coo_matrix( (ones(sum(splitting)),(C_rows,C_rows)), shape=A.shape)
    
    P = C_s_neg.tocsr() + C_s_pos.tocsr() + C_inject.tocsr()

    map = concatenate(([0],cumsum(splitting)))
    P = csr_matrix( (P.data,map[P.indices],P.indptr), shape=(P.shape[0],map[-1]))

    return P




#!/usr/bin/env python


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('pyamg', parent_package, top_path)

    config.add_subpackage('aggregation')
    config.add_subpackage('classical')
    config.add_subpackage('gallery')
    config.add_subpackage('krylov')
    config.add_subpackage('amg_core')
    config.add_subpackage('relaxation')
    config.add_subpackage('testing')
    config.add_subpackage('util')
    config.add_subpackage('vis')

    config.add_data_dir('tests')

    config.make_config_py()
    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



from pyamg.testing import *

from numpy.random import seed, rand

from scipy.sparse import csr_matrix

from pyamg.gallery import poisson, load_example

from pyamg.classical.cr import binormalize, CR

class TestCR(TestCase):
    def setUp(self):
        self.cases = []

        # random matrices
        seed(0)

        for N in [2,3,5]:
            self.cases.append( csr_matrix(rand(N,N)) )

        # Poisson problems in 1D and 2D
        for N in [2,3,5,7,10,11,19]:
            self.cases.append( poisson( (N,), format='csr') )

        for N in [2,3,5,7,10,11]:
            self.cases.append( poisson( (N,N), format='csr') )

        for name in ['knot','airfoil','bar']:
            ex = load_example(name)
            self.cases.append( ex['A'].tocsr() )

    def test_binormalize(self):
        for A in self.cases:
            C = binormalize(A)

            alpha = abs(1.0-C.multiply(C).sum(axis=1)).max()

            assert( alpha < 1e-4 )

    def test_cr(self):
        A = self.cases[6]
        splitting = CR(A)
        assert( splitting.sum() < splitting.shape[0] )



from pyamg.testing import *

from numpy import matrix, array, pi

from pyamg.gallery.diffusion import *

class TestDiffusionStencil2D(TestCase):
    def test_simple_finite_difference(self):
        # isotropic
        stencil = [[ 0.0,-1.0, 0.0],
                   [-1.0, 4.0,-1.0],
                   [ 0.0,-1.0, 0.0]]
        assert_equal(diffusion_stencil_2d(epsilon=1.0, theta=0.0, type='FD'), stencil)

        # weak horizontal
        stencil = [[ 0.0,-1.0, 0.0],
                   [-0.5, 3.0,-0.5],
                   [ 0.0,-1.0, 0.0]]
        assert_almost_equal(diffusion_stencil_2d(epsilon=0.5, theta=0.0, type='FD'), stencil)

        # weak vertical
        stencil = [[ 0.0,-0.5, 0.0],
                   [-1.0, 3.0,-1.0],
                   [ 0.0,-0.5, 0.0]]
        assert_almost_equal(diffusion_stencil_2d(epsilon=0.5, theta=pi/2, type='FD'), stencil)
    
    def test_simple_finite_element(self):
        # isotropic
        stencil = array([[-1.0,-1.0,-1.0],
                         [-1.0, 8.0,-1.0],
                         [-1.0,-1.0,-1.0]]) / 3.0
        assert_almost_equal(diffusion_stencil_2d(epsilon=1.0, theta=0.0, type='FE'), stencil)

        ## weak horizontal
        #assert_almost_equal(diffusion_stencil_2d(epsilon=0.5, theta=0.0, type='FE'), stencil)

        ## weak vertical
        #assert_almost_equal(diffusion_stencil_2d(epsilon=0.5, theta=pi/2, type='FE'), stencil)

    def test_zero_sum(self):
        """test that stencil entries sum to zero"""
        for type in ['FD','FE']:
            for theta in [pi/8, pi/5, pi/4, pi/3, pi/2, pi]:
                for epsilon in [0.001, 0.01, 1.0]:
                    stencil = diffusion_stencil_2d(epsilon=epsilon, theta=theta, type=type)
                    assert_almost_equal(stencil.sum(), 0.0)


    def test_rotation_invariance(self):
        """test invariance to theta when epsilon=1.0"""

        for type in ['FD','FE']:
            expected = diffusion_stencil_2d(epsilon=1.0, theta=0.0, type=type)
            for theta in [pi/8, pi/4, pi/3, pi/2, pi]:
                result = diffusion_stencil_2d(epsilon=1.0, theta=theta, type=type)
                assert_almost_equal(result, expected)




from pyamg.testing import *

import numpy
from numpy import ones, eye, zeros, bincount, empty, asarray, array
from numpy.random import seed
from scipy import rand
from scipy.sparse import csr_matrix, csc_matrix, coo_matrix

from pyamg.gallery import poisson, load_example
from pyamg.graph import *
from pyamg import amg_core

def canonical_graph(G):
    # convert to expected format
    # - remove diagonal entries
    # - all nonzero values = 1
    G = coo_matrix(G)
    
    mask = G.row != G.col
    G.row     = G.row[mask]
    G.col     = G.col[mask]
    G.data    = G.data[mask]
    G.data[:] = 1
    return G

def assert_is_mis(G,mis):
    G = canonical_graph(G)

    # no MIS vertices joined by an edge
    if G.nnz > 0:
        assert( (mis[G.row] + mis[G.col]).max() <= 1 )
    # all non-set vertices have set neighbor
    assert( (mis + G*mis).min() == 1 )

def assert_is_vertex_coloring(G,c):
    G = canonical_graph(G)

    # no colors joined by an edge
    assert( (c[G.row] != c[G.col]).all() )
    # all colors up to K occur at least once
    assert( (bincount(c) > 0).all() )


class TestGraph(TestCase):
    def setUp(self):
        cases = []
        seed(0)

        for i in range(5):
            A = rand(8,8) > 0.5
            cases.append( canonical_graph(A + A.T).astype(float) )

        cases.append( zeros((1,1)) )
        cases.append( zeros((2,2)) )
        cases.append( zeros((8,8)) )
        cases.append( ones((2,2)) - eye(2) )
        cases.append( poisson( (5,) ) )
        cases.append( poisson( (5,5) ) )
        cases.append( poisson( (11,11) ) )
        cases.append( poisson( (5,5,5) ) )
        for name in ['airfoil','bar','knot']:
            cases.append( load_example(name)['A'] )

        cases = [ canonical_graph(G) for G in cases ]

        self.cases = cases        

    def test_maximal_independent_set(self):
        # test that method works with diagonal entries
        assert_equal( maximal_independent_set(eye(2)), [1, 1] )

        for algo in ['serial','parallel']:
            for G in self.cases:
                mis = maximal_independent_set(G, algo=algo)
                assert_is_mis(G,mis)
        
        for G in self.cases:
            for k in [1,2,3,4]:
                mis = maximal_independent_set(G, k=k)
                if k > 1:
                    G = (G + eye(G.shape[0]))**k
                    G = canonical_graph(G)
                assert_is_mis(G, mis)
    
    def test_vertex_coloring(self):
        # test that method works with diagonal entries
        assert_equal( vertex_coloring(eye(1)), [0] )
        assert_equal( vertex_coloring(eye(3)), [0,0,0] )
        assert_equal( sorted(vertex_coloring(ones((3,3)))), [0,1,2] )

        for method in ['MIS','JP','LDF']:
            for G in self.cases:
                c = vertex_coloring(G, method=method)
                assert_is_vertex_coloring(G,c)
    
    def test_bellman_ford(self):
        numpy.random.seed(0)

        for G in self.cases:
            G.data = rand(G.nnz)
            N = G.shape[0]

            for n_seeds in [ N/20, N/10, N-2, N]:
                if n_seeds > G.shape[0] or n_seeds < 1: 
                    continue
    
                seeds = numpy.random.permutation(N)[:n_seeds]
                D_expected,S_expected = reference_bellman_ford(G, seeds) 
                D_result,S_result     = bellman_ford(G, seeds)
    
                assert_equal(D_result, D_expected)
                assert_equal(S_result, S_expected)

    def test_lloyd_cluster(self):
        numpy.random.seed(0)

        for G in self.cases:
            G.data = rand(G.nnz)
            
            for n_seeds in [5]:
                if n_seeds > G.shape[0]: 
                    continue
    
                distances,clusters,centers = lloyd_cluster(G, n_seeds)

class TestComplexGraph(TestCase):
    def setUp(self):
        cases = []
        seed(0)

        for i in range(5):
            A = rand(8,8) > 0.5
            cases.append( canonical_graph(A + A.T).astype(float) )

        cases = [ canonical_graph(G)+1.0j*canonical_graph(G) for G in cases ]

        self.cases = cases        

    def test_maximal_independent_set(self):
        # test that method works with diagonal entries
        assert_equal( maximal_independent_set(eye(2)), [1, 1] )

        for algo in ['serial','parallel']:
            for G in self.cases:
                mis = maximal_independent_set(G, algo=algo)
                assert_is_mis(G,mis)
        
    def test_vertex_coloring(self):
        for method in ['MIS','JP','LDF']:
            for G in self.cases:
                c = vertex_coloring(G, method=method)
                assert_is_vertex_coloring(G,c)
    
    def test_lloyd_cluster(self):
        numpy.random.seed(0)

        for G in self.cases:
            G.data = rand(G.nnz) + 1.0j*rand(G.nnz)
            
            for n_seeds in [5]:
                if n_seeds > G.shape[0]: 
                    continue
    
                distances,clusters,centers = lloyd_cluster(G, n_seeds)



class TestVertexColorings(TestCase):
    def setUp(self):
        #      3---4
        #    / | / |
        #  0---1---2
        G0 = array([[ 0, 1, 0, 1, 0],
                    [ 1, 0, 1, 1, 1],
                    [ 0, 1, 0, 0, 1],
                    [ 1, 1, 0, 0, 1],
                    [ 0, 1, 1, 1, 0]])
        self.G0 = csr_matrix(G0) 
        assert_equal( (self.G0 - self.G0.T).nnz, 0) # make sure graph is symmetric

        #  2        5
        #  | \    / | 
        #  0--1--3--4
        G1 = array([[ 0, 1, 1, 0, 0, 0],
                    [ 1, 0, 1, 1, 0, 0],
                    [ 1, 1, 0, 0, 0, 0],
                    [ 0, 1, 0, 0, 1, 1],
                    [ 0, 0, 0, 1, 0, 1],
                    [ 0, 0, 0, 1, 1, 0]])
        self.G1 = csr_matrix(G1) 
        assert_equal( (self.G1 - self.G1.T).nnz, 0) # make sure graph is symmetric

    def test_vertex_coloring_JP(self):
        fn = amg_core.vertex_coloring_jones_plassmann

        weights  = array([0.8, 0.1, 0.9, 0.7, 0.6], dtype='float64')
        coloring = empty(5, dtype='intc')
        fn(self.G0.shape[0], self.G0.indptr, self.G0.indices, coloring, weights)
        assert_equal( coloring, [2, 0, 1, 1, 2] )
        
        weights  = array([0.1, 0.2, 0.3, 0.1, 0.2, 0.3], dtype='float64')
        coloring = empty(6, dtype='intc')
        fn(self.G1.shape[0], self.G1.indptr, self.G1.indices, coloring, weights)
        assert_equal( coloring, [2, 0, 1, 1, 2, 0] )

    def test_vertex_coloring_LDF(self):
        fn = amg_core.vertex_coloring_LDF

        weights  = array([0.8, 0.1, 0.9, 0.7, 0.6], dtype='float64')
        coloring = empty(5, dtype='intc')
        fn(self.G0.shape[0], self.G0.indptr, self.G0.indices, coloring, weights)
        assert_equal( coloring, [2, 0, 1, 1, 2] )

        weights  = array([0.1, 0.2, 0.3, 0.1, 0.2, 0.3], dtype='float64')
        coloring = empty(6, dtype='intc')
        fn(self.G1.shape[0], self.G1.indptr, self.G1.indices, coloring, weights)
        assert_equal( coloring, [2, 0, 1, 2, 1, 0] )



def test_breadth_first_search():
    from pyamg.graph import breadth_first_search

    BFS = breadth_first_search

    G = csr_matrix([[0,1,0,0],
                    [1,0,1,0],
                    [0,1,0,1],
                    [0,0,1,0]])

    assert_equal(BFS(G,0)[1], [0,1,2,3])
    assert_equal(BFS(G,1)[1], [1,0,1,2])
    assert_equal(BFS(G,2)[1], [2,1,0,1])
    assert_equal(BFS(G,3)[1], [3,2,1,0])

    G = csr_matrix([[0,1,0,0],
                    [1,0,1,0],
                    [0,1,0,0],
                    [0,0,0,0]])

    assert_equal(BFS(G,0)[1], [ 0, 1, 2,-1])
    assert_equal(BFS(G,1)[1], [ 1, 0, 1,-1])
    assert_equal(BFS(G,2)[1], [ 2, 1, 0,-1])
    assert_equal(BFS(G,3)[1], [-1,-1,-1, 0])


def test_connected_components():

    cases = []
    cases.append( csr_matrix([[0,1,0,0],
                              [1,0,1,0],
                              [0,1,0,1],
                              [0,0,1,0]]) )

    cases.append( csr_matrix([[0,1,0,0],
                              [1,0,0,0],
                              [0,0,0,1],
                              [0,0,1,0]]) )
    
    cases.append( csr_matrix([[0,1,0,0],
                              [1,0,0,0],
                              [0,0,0,0],
                              [0,0,0,0]]) )
    
    cases.append( csr_matrix([[0,0,0,0],
                              [0,0,0,0],
                              [0,0,0,0],
                              [0,0,0,0]]) )

    #  2        5
    #  | \    / | 
    #  0--1--3--4
    cases.append( csr_matrix([[ 0, 1, 1, 0, 0, 0],
                              [ 1, 0, 1, 1, 0, 0],
                              [ 1, 1, 0, 0, 0, 0],
                              [ 0, 1, 0, 0, 1, 1],
                              [ 0, 0, 0, 1, 0, 1],
                              [ 0, 0, 0, 1, 1, 0]]) )
    
    #  2        5
    #  | \    / | 
    #  0  1--3--4
    cases.append( csr_matrix([[ 0, 0, 1, 0, 0, 0],
                              [ 0, 0, 1, 1, 0, 0],
                              [ 1, 1, 0, 0, 0, 0],
                              [ 0, 1, 0, 0, 1, 1],
                              [ 0, 0, 0, 1, 0, 1],
                              [ 0, 0, 0, 1, 1, 0]]) )

    #  2        5
    #  | \    / | 
    #  0--1  3--4
    cases.append( csr_matrix([[ 0, 1, 1, 0, 0, 0],
                              [ 1, 0, 1, 0, 0, 0],
                              [ 1, 1, 0, 0, 0, 0],
                              [ 0, 0, 0, 0, 1, 1],
                              [ 0, 0, 0, 1, 0, 1],
                              [ 0, 0, 0, 1, 1, 0]]) )



    #######################################
    # Compare to reference implementation #
    #######################################

    for G in cases:
        result = connected_components(G)

        assert_equal(result.min(), 0)
        
        def array_to_set_of_sets(arr):
            """convert array to set of sets format"""
            D = {}
            for i in set(arr):
                D[i] = set()
            for n,i in enumerate(arr):
                D[i].add(n)
            return set([frozenset(s) for s in D.values()])
        
        result = array_to_set_of_sets(result)
        expected = reference_connected_components(G)

        assert_equal(result, expected)

        
def test_complex_connected_components():

    cases = []
    cases.append( csr_matrix([[0,1,0,0],
                              [1,0,1,0],
                              [0,1,0,1],
                              [0,0,1,0]]) )

    cases.append( csr_matrix([[0,1,0,0],
                              [1,0,0,0],
                              [0,0,0,1],
                              [0,0,1,0]]) )
    
    cases.append( csr_matrix([[0,1,0,0],
                              [1,0,0,0],
                              [0,0,0,0],
                              [0,0,0,0]]) )
    
    cases.append( csr_matrix([[0,0,0,0],
                              [0,0,0,0],
                              [0,0,0,0],
                              [0,0,0,0]]) )

    #  2        5
    #  | \    / | 
    #  0--1--3--4
    cases.append( csr_matrix([[ 0, 1, 1, 0, 0, 0],
                              [ 1, 0, 1, 1, 0, 0],
                              [ 1, 1, 0, 0, 0, 0],
                              [ 0, 1, 0, 0, 1, 1],
                              [ 0, 0, 0, 1, 0, 1],
                              [ 0, 0, 0, 1, 1, 0]]) )
    
    #  2        5
    #  | \    / | 
    #  0  1--3--4
    cases.append( csr_matrix([[ 0, 0, 1, 0, 0, 0],
                              [ 0, 0, 1, 1, 0, 0],
                              [ 1, 1, 0, 0, 0, 0],
                              [ 0, 1, 0, 0, 1, 1],
                              [ 0, 0, 0, 1, 0, 1],
                              [ 0, 0, 0, 1, 1, 0]]) )

    #  2        5
    #  | \    / | 
    #  0--1  3--4
    cases.append( csr_matrix([[ 0, 1, 1, 0, 0, 0],
                              [ 1, 0, 1, 0, 0, 0],
                              [ 1, 1, 0, 0, 0, 0],
                              [ 0, 0, 0, 0, 1, 1],
                              [ 0, 0, 0, 1, 0, 1],
                              [ 0, 0, 0, 1, 1, 0]]) )

    # Create complex data entries
    cases = [ G+1.0j*G for G in cases ]

    #######################################
    # Compare to reference implementation #
    #######################################

    for G in cases:
        result = connected_components(G)

        assert_equal(result.min(), 0)
        
        def array_to_set_of_sets(arr):
            """convert array to set of sets format"""
            D = {}
            for i in set(arr):
                D[i] = set()
            for n,i in enumerate(arr):
                D[i].add(n)
            return set([frozenset(s) for s in D.values()])
        
        result = array_to_set_of_sets(result)
        expected = reference_connected_components(G)

        assert_equal(result, expected)
       

#############################
# reference implementations #
#############################

def reference_connected_components(G):
    G = G.tocsr()
    N = G.shape[0]

    def DFS(i, G, component, visited):
        if i not in visited:
            component.add(i)
            visited.add(i)
            for j in G.indices[G.indptr[i]:G.indptr[i+1]]:
                DFS(j, G, component, visited)
    

    num_components = 0
    visited = set()
    components = set()
    for i in range(N):
        if i not in visited:
            component = set()
            DFS(i, G, component, visited)
            components.add(frozenset(component))

    return components            

from pyamg.graph import max_value
def reference_bellman_ford(G,seeds):
    G = G.tocoo()
    N = G.shape[0]
   
    seeds = asarray(seeds, dtype='intc')

    distances        = empty( N, dtype=G.dtype )
    distances[:]     = max_value(G.dtype)
    distances[seeds] = 0

    nearest_seed        = empty(N, dtype='intc')
    nearest_seed[:]     = -1
    nearest_seed[seeds] = seeds

    while True:
        update = False

        for (i,j,v) in zip(G.row,G.col,G.data):

            if distances[j] + v < distances[i]:
                update = True
                distances[i]    = distances[j] + v
                nearest_seed[i] = nearest_seed[j]

        if not update:
            break

    return (distances, nearest_seed) 





"""
Script for testing the installation.

Mainly used for Travis-CI.

Can be expand tof more verbose testing and coverage.

See NumPy/SciPy examples.
"""
import sys

sys.path.pop(0)

import pyamg

result = pyamg.test()

print result

print "--------"
print result.wasSuccessful()

if result.wasSuccessful():
    sys.exit(0)
else:
    sys.exit(1)



"""Method to create pre- and post-smoothers on the levels of a multilevel_solver
"""

import numpy
import scipy
import relaxation
from chebyshev import chebyshev_polynomial_coefficients
from pyamg.util.utils import scale_rows, get_block_diag, UnAmal, get_diagonal
from pyamg.util.linalg import approximate_spectral_radius
from pyamg.krylov import gmres, cgne, cgnr, cg
from pyamg import amg_core
from scipy.linalg import lapack as la

__docformat__ = "restructuredtext en"

__all__ = ['change_smoothers']

def unpack_arg(v):
    if isinstance(v,tuple):
        return v[0],v[1]
    else:
        return v,{}

def change_smoothers(ml, presmoother, postsmoother):
    '''
    Initialize pre- and post- smoothers throughout a multilevel_solver, with
    the option of having different smoothers at different levels

    For each level of the multilevel_solver 'ml' (except the coarsest level),
    initialize the .presmoother() and .postsmoother() methods used in the 
    multigrid cycle.

    Parameters
    ----------
    ml : {pyamg multilevel hierarchy}
        Data structure that stores the multigrid hierarchy.
    presmoother : {None, string, tuple, list}
        presmoother can be (1) the name of a supported smoother, 
        e.g. "gauss_seidel", (2) a tuple of the form ('method','opts') where 
        'method' is the name of a supported smoother and 'opts' a dict of
        keyword arguments to the smoother, or (3) a list of instances of options 1 or 2.
        See the Examples section for illustrations of the format. 

        If presmoother is a list, presmoother[i] determines the smoothing
        strategy for level i.  Else, presmoother defines the same strategy 
        for all levels.
        
        If len(presmoother) < len(ml.levels), then 
        presmoother[-1] is used for all remaining levels
        
        If len(presmoother) > len(ml.levels), then
        the remaining smoothing strategies are ignored
    
    postsmoother : {string, tuple, list}
        Defines postsmoother in identical fashion to presmoother

    Returns
    -------
    ml changed in place
    ml.level[i].presmoother  <===  presmoother[i]
    ml.level[i].postsmoother  <===  postsmoother[i]

    Notes
    -----
    - Parameter 'omega' of the Jacobi, Richardson, and jacobi_ne
      methods is scaled by the spectral radius of the matrix on 
      each level.  Therefore 'omega' should be in the interval (0,2).
    - Parameter 'withrho' (default: True) controls whether the omega is
      rescaled by the spectral radius in jacobi, block_jacobi, and jacobi_ne
    - By initializing the smoothers after the hierarchy has been setup, allows
      for "algebraically" directed relaxation, such as strength_based_schwarz,
      which uses only the strong connections of a degree-of-freedom to define
      overlapping regions
    - Available smoother methods::

        gauss_seidel
        block_gauss_seidel
        jacobi
        block_jacobi
        richardson
        sor
        chebyshev
        gauss_seidel_nr
        gauss_seidel_ne
        jacobi_ne
        cg
        gmres
        cgne
        cgnr
        schwarz
        strength_based_schwarz
        None

    Examples
    --------
    >>> from pyamg.gallery import poisson
    >>> from pyamg.aggregation import smoothed_aggregation_solver
    >>> from pyamg.relaxation.smoothing import change_smoothers
    >>> from pyamg.util.linalg import norm
    >>> from scipy import rand, array, mean
    >>> A = poisson((10,10), format='csr')
    >>> b = rand(A.shape[0],)
    >>> ml = smoothed_aggregation_solver(A, max_coarse=10)
    >>> #
    >>> ## Set all levels to use gauss_seidel's defaults
    >>> smoothers = 'gauss_seidel'
    >>> change_smoothers(ml, presmoother=smoothers, postsmoother=smoothers)
    >>> residuals=[]
    >>> x = ml.solve(b, tol=1e-8, residuals=residuals)
    >>> #
    >>> ## Set all levels to use three iterations of gauss_seidel's defaults
    >>> smoothers = ('gauss_seidel', {'iterations' : 3})
    >>> change_smoothers(ml, presmoother=smoothers, postsmoother=None)
    >>> residuals=[]
    >>> x = ml.solve(b, tol=1e-8, residuals=residuals)
    >>> #
    >>> ## Set level 0 to use gauss_seidel's defaults, and all  
    >>> ## subsequent levels to use 5 iterations of cgnr
    >>> smoothers = ['gauss_seidel', ('cgnr', {'maxiter' : 5})]
    >>> change_smoothers(ml, presmoother=smoothers, postsmoother=smoothers)
    >>> residuals=[]
    >>> x = ml.solve(b, tol=1e-8, residuals=residuals)
    '''

    # interpret arguments into list
    if isinstance(presmoother, str) or isinstance(presmoother, tuple) or (presmoother is None):
        presmoother = [presmoother]
    elif not isinstance(presmoother, list):
        raise ValueError,'Unrecognized presmoother'

    if isinstance(postsmoother, str) or isinstance(postsmoother, tuple) or (postsmoother is None):
        postsmoother = [postsmoother]
    elif not isinstance(postsmoother, list):
        raise ValueError,'Unrecognized postsmoother'

    # set ml.levels[i].presmoother = presmoother[i]
    i = 0
    for i in range( min(len(presmoother), len(ml.levels[:-1])) ):
        # unpack presmoother[i]
        fn,kwargs = unpack_arg(presmoother[i])
        # get function handle
        try:
            setup_presmoother = eval('setup_' + str(fn))
        except NameError, ne:
            raise NameError("invalid presmoother method: ", fn)
        ml.levels[i].presmoother = setup_presmoother(ml.levels[i], **kwargs)
    
    # Fill in remaining levels
    for j in range(i+1, len(ml.levels[:-1])):
        ml.levels[j].presmoother = setup_presmoother(ml.levels[j], **kwargs)


    # set ml.levels[i].postsmoother = postsmoother[i]
    i = 0
    for i in range( min(len(postsmoother), len(ml.levels[:-1])) ):
        # unpack postsmoother[i]
        fn,kwargs = unpack_arg(postsmoother[i])
        # get function handle
        try:
            setup_postsmoother = eval('setup_' + str(fn))
        except NameError, ne:
            raise NameError("invalid postsmoother method: ", fn)
        ml.levels[i].postsmoother = setup_postsmoother(ml.levels[i], **kwargs)
    
    # Fill in remaining levels
    for j in range(i+1, len(ml.levels[:-1])):
        ml.levels[j].postsmoother = setup_postsmoother(ml.levels[j], **kwargs)

def rho_D_inv_A(A):
    """
    Return the (approx.) spectral radius of D^-1 * A 
    
    Parameters
    ----------
    A : {sparse-matrix}

    Returns
    -------
    approximate spectral radius of diag(A)^{-1} A

    Examples
    --------
    >>> from pyamg.gallery import poisson
    >>> from pyamg.relaxation.smoothing import rho_D_inv_A
    >>> from scipy.sparse import csr_matrix
    >>> import numpy
    >>> A = csr_matrix(numpy.array([[1.0,0,0],[0,2.0,0],[0,0,3.0]]))
    >>> print rho_D_inv_A(A)
    1.0
    """
    
    if not hasattr(A, 'rho_D_inv'):
        D_inv = get_diagonal(A, inv=True)
        D_inv_A = scale_rows(A, D_inv, copy=True)
        A.rho_D_inv = approximate_spectral_radius(D_inv_A)
    
    return A.rho_D_inv


def rho_block_D_inv_A(A, Dinv):
    """
    Return the (approx.) spectral radius of block D^-1 * A 
    
    Parameters
    ----------
    A : {sparse-matrix}
        size NxN
    Dinv : {array}
        Inverse of diagonal blocks of A
        size (N/blocksize, blocksize, blocksize)

    Returns
    -------
    approximate spectral radius of (Dinv A)

    Examples
    --------
    >>> from pyamg.gallery import poisson
    >>> from pyamg.relaxation.smoothing import rho_block_D_inv_A
    >>> from pyamg.util.utils import get_block_diag
    >>> import numpy
    >>> A = poisson((10,10), format='csr')
    >>> Dinv = get_block_diag(A, blocksize=4, inv_flag=True)
    
    """

    if not hasattr(A, 'rho_block_D_inv'):
        from scipy.sparse.linalg import LinearOperator
        
        blocksize = Dinv.shape[1]
        if Dinv.shape[1] != Dinv.shape[2]:
            raise ValueError('Dinv has incorrect dimensions')
        elif Dinv.shape[0] != A.shape[0]/blocksize:
            raise ValueError('Dinv and A have incompatible dimensions')
        
        Dinv = scipy.sparse.bsr_matrix( (Dinv, \
                scipy.arange(Dinv.shape[0]), scipy.arange(Dinv.shape[0]+1)), shape=A.shape)
        
        # Don't explicitly form Dinv*A
        def matvec(x):
            return Dinv*(A*x)
        D_inv_A = LinearOperator(A.shape, matvec, dtype=A.dtype)
        
        A.rho_block_D_inv = approximate_spectral_radius(D_inv_A)

    return A.rho_block_D_inv

def matrix_asformat(lvl, name, format, blocksize=None):
    '''
    This routine looks for the matrix "name" in the specified format as a
    member of the level instance, lvl.  For example, if name='A', format='bsr'
    and blocksize=(4,4), and if lvl.Absr44 exists with the correct blocksize,
    then lvl.Absr is returned.  If the matrix doesn't already exist, lvl.name
    is converted to the desired format, and made a member of lvl.
    
    Only create such persistent copies of a matrix for routines such as
    presmoothing and postsmoothing, where the matrix conversion is done every
    cycle.

    Calling this function can _dramatically_ increase your memory costs.
    Be careful with it's usage.
    '''
    
    desired_matrix = 'lvl.' + name + format
    if format == 'bsr':
        desired_matrix +=  str(blocksize[0])+str(blocksize[1])
    base_matrix = 'lvl.' + name

    if hasattr(lvl, desired_matrix[4:]):
        # if lvl already contains lvl.name+format
        pass
    elif eval(base_matrix).format == format and format != 'bsr':
        # is base_matrix already in the correct format?
        exec desired_matrix +' = '+ base_matrix
    elif eval(base_matrix).format == format and format == 'bsr':
        # make sure blocksize is correct
        if eval(base_matrix).blocksize != blocksize:
            exec desired_matrix +' = '+ base_matrix+'.to'+format+'(blocksize='+str(blocksize)+')'
        else:
            exec desired_matrix +' = '+ base_matrix
    elif format == 'bsr':
        # convert
        exec desired_matrix +' = '+ base_matrix+'.to'+format+'(blocksize='+str(blocksize)+')'
    else: 
        # convert
        exec desired_matrix + ' = lvl.' + name + '.to' + format + '()'

    return eval(desired_matrix)


#Appends
#A.schwarz_tuple = (subdomain, subdomain_ptr, inv_subblock, inv_subblock_ptr) to
#the matrix passed in If this tuple already exists, return it, otherwise compute
#it Should work for passing in both A and the Strength matrix C Make sure to
#wrap an Acsr into the schwarz call

#make sure that it handles passing in preset subdomain stuff, and recomputing it

#If subdomain and subdomain_ptr are passed in, check to see that they are the
#same as any preexisting subdomain and subdomain_ptr?

def schwarz_parameters(A, subdomain=None, subdomain_ptr=None, 
                       inv_subblock=None, inv_subblock_ptr=None):
    '''
    Helper function for setting up Schwarz relaxation.  This function avoids
    recomputing the subdomains and block inverses manytimes, e.g., it avoids
    a costly double computation when setting up pre and post smoothing with Schwarz.
    
    Parameters
    ----------
    A {csr_matrix}

    Returns
    -------
    A.schwarz_parameters[0] is subdomain
    A.schwarz_parameters[1] is subdomain_ptr
    A.schwarz_parameters[2] is inv_subblock
    A.schwarz_parameters[3] is inv_subblock_ptr
    '''
    
    # Check if A has a pre-existing set of Schwarz parameters
    if hasattr(A, 'schwarz_parameters'):
        if subdomain is not None and subdomain_ptr is not None:
            # check that the existing parameters correspond to the same subdomains
            if numpy.array(A.schwarz_parameters[0] == subdomain).all() and \
               numpy.array(A.schwarz_parameters[1] == subdomain_ptr).all():
                return A.schwarz_parameters
        else:
            return A.schwarz_parameters
    
    # Default is to use the overlapping regions defined by A's sparsity pattern
    if subdomain is None or subdomain_ptr is None:
        subdomain_ptr = A.indptr.copy()
        subdomain = A.indices.copy()

    ##
    # Extract each subdomain's block from the matrix
    if inv_subblock is None or inv_subblock_ptr is None:
        inv_subblock_ptr = numpy.zeros(subdomain_ptr.shape, dtype=A.indices.dtype)
        blocksize = (subdomain_ptr[1:] - subdomain_ptr[:-1])
        inv_subblock_ptr[1:] = numpy.cumsum(blocksize*blocksize)
        
        ##
        # Extract each block column from A
        inv_subblock = numpy.zeros((inv_subblock_ptr[-1],), dtype=A.dtype)
        amg_core.extract_subblocks(A.indptr, A.indices, A.data, inv_subblock, 
                          inv_subblock_ptr, subdomain, subdomain_ptr, 
                          int(subdomain_ptr.shape[0]-1), A.shape[0])
        ##
        # Choose tolerance for which singular values are zero in *gelss below
        t = A.dtype.char
        eps = numpy.finfo(numpy.float).eps
        feps = numpy.finfo(numpy.single).eps
        geps = numpy.finfo(numpy.longfloat).eps
        _array_precision = {'f': 0, 'd': 1, 'g': 2, 'F': 0, 'D': 1, 'G':2}
        cond = {0: feps*1e3, 1: eps*1e6, 2: geps*1e6}[_array_precision[t]]

        ##
        # Invert each block column
        my_pinv, = la.get_lapack_funcs(['gelss'], (numpy.ones((1,), dtype=A.dtype)) )
        for i in xrange(subdomain_ptr.shape[0]-1):
            m = blocksize[i]
            rhs = scipy.eye(m,m, dtype=A.dtype)
            gelssoutput = my_pinv(inv_subblock[inv_subblock_ptr[i]:inv_subblock_ptr[i+1]].reshape(m,m), 
                            rhs, cond=cond, overwrite_a=True, overwrite_b=True)
            inv_subblock[inv_subblock_ptr[i]:inv_subblock_ptr[i+1]] = numpy.ravel(gelssoutput[1])

    A.schwarz_parameters = (subdomain, subdomain_ptr, inv_subblock, inv_subblock_ptr)
    return A.schwarz_parameters

"""
    The following setup_smoother_name functions are helper functions for
    parsing user input and assigning each level the appropriate smoother for
    the above functions "change_smoothers".

    The standard interface is

    Parameters
    ----------
    lvl : {multilevel level}
        the level in the hierarchy for which to assign a smoother
    iterations : {int}
        how many smoother iterations
    optional_params : {}
        optional params specific for each method such as omega or sweep

    Returns
    -------
    Function pointer for the appropriate relaxation method for level=lvl

    Examples
    --------
    See change_smoothers above

"""
def setup_gauss_seidel(lvl, iterations=1, sweep='forward'):
    def smoother(A,x,b):
        relaxation.gauss_seidel(A, x, b, iterations=iterations, sweep=sweep)
    return smoother
        
def setup_jacobi(lvl, iterations=1, omega=1.0, withrho=True):
    if withrho:
        omega = omega/rho_D_inv_A(lvl.A)
    def smoother(A,x,b):
        relaxation.jacobi(A, x, b, iterations=iterations, omega=omega)
    return smoother

def setup_schwarz(lvl, iterations=1, subdomain=None, subdomain_ptr=None, \
                  inv_subblock=None, inv_subblock_ptr=None, sweep='symmetric'):

    Acsr = matrix_asformat(lvl, 'A', 'csr')
    subdomain, subdomain_ptr, inv_subblock, inv_subblock_ptr = \
            schwarz_parameters(Acsr, subdomain, subdomain_ptr, inv_subblock, inv_subblock_ptr)

    def smoother(A,x,b):
        relaxation.schwarz(A, x, b, iterations=iterations, subdomain=subdomain, \
                           subdomain_ptr=subdomain_ptr, inv_subblock=inv_subblock,\
                           inv_subblock_ptr=inv_subblock_ptr, sweep=sweep)
    return smoother


def setup_strength_based_schwarz(lvl, iterations=1, sweep='symmetric'):
    # Use the overlapping regions defined by the strength of connection matrix C 
    # for the overlapping Schwarz method
    if not hasattr(lvl,'C'):
        C = lvl.A.tocsr()
    else:
        C = lvl.C.tocsr()

    subdomain_ptr = C.indptr.copy()
    subdomain = C.indices.copy()

    return setup_schwarz(lvl, iterations=iterations, subdomain=subdomain, \
                         subdomain_ptr=subdomain_ptr, sweep=sweep)

def setup_block_jacobi(lvl, iterations=1, omega=1.0, Dinv=None, blocksize=None, withrho=True):
    ##
    # Determine Blocksize
    if blocksize is None and Dinv is None:
        if scipy.sparse.isspmatrix_csr(lvl.A):
            blocksize = 1
        elif scipy.sparse.isspmatrix_bsr(lvl.A):
            blocksize = lvl.A.blocksize[0]
    elif blocksize is None:
        blocksize = Dinv.shape[1]
    
    if blocksize == 1:
        # Block Jacobi is equivalent to normal Jacobi
        return setup_jacobi(lvl, iterations=iterations, omega=omega, withrho=withrho)
    else:
        # Use Block Jacobi
        if Dinv is None:
            Dinv = get_block_diag(lvl.A, blocksize=blocksize, inv_flag=True)
        if withrho:
            omega = omega/rho_block_D_inv_A(lvl.A, Dinv)
        def smoother(A,x,b):
            relaxation.block_jacobi(A, x, b, iterations=iterations, omega=omega, \
                                    Dinv=Dinv, blocksize=blocksize)
        return smoother

def setup_block_gauss_seidel(lvl, iterations=1, sweep='forward', Dinv=None, blocksize=None):
    ##
    # Determine Blocksize
    if blocksize is None and Dinv is None:
        if scipy.sparse.isspmatrix_csr(lvl.A):
            blocksize = 1
        elif scipy.sparse.isspmatrix_bsr(lvl.A):
            blocksize = lvl.A.blocksize[0]
    elif blocksize is None:
        blocksize = Dinv.shape[1]

    if blocksize == 1:
        # Block GS is equivalent to normal GS
        return setup_gauss_seidel(lvl, iterations=iterations, sweep=sweep)
    else:
        # Use Block GS       
        if Dinv is None:
            Dinv = get_block_diag(lvl.A, blocksize=blocksize, inv_flag=True)
        def smoother(A,x,b):
            relaxation.block_gauss_seidel(A, x, b, iterations=iterations, \
                               Dinv=Dinv, blocksize=blocksize, sweep=sweep)

        return smoother

def setup_richardson(lvl, iterations=1, omega=1.0):
    omega = omega/approximate_spectral_radius(lvl.A)
    def smoother(A,x,b):
        relaxation.polynomial(A, x, b, coefficients=[omega], iterations=iterations)
    return smoother

def setup_sor(lvl, omega=0.5, iterations=1, sweep='forward'):
    def smoother(A,x,b):
        relaxation.sor(A, x, b, omega=omega, iterations=iterations, sweep=sweep)
    return smoother

def setup_chebyshev(lvl, lower_bound=1.0/30.0, upper_bound=1.1, degree=3, iterations=1):
    rho = approximate_spectral_radius(lvl.A)
    a = rho * lower_bound
    b = rho * upper_bound
    coefficients = -chebyshev_polynomial_coefficients(a, b, degree)[:-1] # drop the constant coefficient
    def smoother(A,x,b):
        relaxation.polynomial(A, x, b, coefficients=coefficients, iterations=iterations)
    return smoother

def setup_jacobi_ne(lvl, iterations=1, omega=1.0, withrho=True):
    Acsr = matrix_asformat(lvl, 'A', 'csr')
    if withrho:
        omega = omega/rho_D_inv_A(Acsr)**2
    def smoother(A,x,b):
        relaxation.jacobi_ne(Acsr, x, b, iterations=iterations, omega=omega)
    return smoother

def setup_gauss_seidel_ne(lvl, iterations=1, sweep='forward', omega=1.0):
    Acsr = matrix_asformat(lvl, 'A', 'csr')
    def smoother(A,x,b):
        relaxation.gauss_seidel_ne(Acsr, x, b, iterations=iterations, sweep=sweep, omega=omega)
    return smoother

def setup_gauss_seidel_nr(lvl, iterations=1, sweep='forward', omega=1.0):
    Acsc = matrix_asformat(lvl, 'A', 'csc')
    def smoother(A,x,b):
        relaxation.gauss_seidel_nr(Acsc, x, b, iterations=iterations, sweep=sweep, omega=omega)
    return smoother

def setup_gmres(lvl, tol=1e-12, maxiter=1, restrt=None, M=None, callback=None, residuals=None):
    def smoother(A,x,b):
        x[:] = (gmres(A, b, x0=x, tol=tol, maxiter=maxiter, restrt=restrt, M=M, callback=callback, residuals=residuals)[0]).reshape(x.shape)
    return smoother
            
def setup_cg(lvl, tol=1e-12, maxiter=1, M=None, callback=None, residuals=None):
    def smoother(A,x,b):
        x[:] = (cg(A, b, x0=x, tol=tol, maxiter=maxiter, M=M, callback=callback, residuals=residuals)[0]).reshape(x.shape)
    return smoother

def setup_cgne(lvl, tol=1e-12, maxiter=1, M=None, callback=None, residuals=None):
    def smoother(A,x,b):
        x[:] = (cgne(A, b, x0=x, tol=tol, maxiter=maxiter, M=M, callback=callback, residuals=residuals)[0]).reshape(x.shape)
    return smoother

def setup_cgnr(lvl, tol=1e-12, maxiter=1, M=None, callback=None, residuals=None):
    def smoother(A,x,b):
        x[:] = (cgnr(A, b, x0=x, tol=tol, maxiter=maxiter, M=M, callback=callback, residuals=residuals)[0]).reshape(x.shape)
    return smoother

def setup_None(lvl):
    def smoother(A,x,b):
        pass
    return smoother





from pyamg.testing import *

from pyamg.gallery.example import *

class TestExample(TestCase):
    def test_load_example(self):
        knot = load_example('knot')

        A        = knot['A']
        B        = knot['B']
        vertices = knot['vertices']
        elements = knot['elements']

        assert_equal( A.shape, (239,239) )
        assert_equal( B.shape, (239,1) )
        assert_equal( vertices.shape, (240,3) )
        assert_equal( elements.shape, (480,3) )




from pyamg.testing import *

from scipy import matrix

from numpy import array, abs
from scipy import real, imag
from scipy.linalg import eigvals
from pyamg.gallery.laplacian import *

class TestPoisson(TestCase):
    def test_poisson(self):
        cases = []

        # 1D
        cases.append( ((1,),matrix([[2]])) )
        cases.append( ((2,),matrix([[ 2,-1],
                                    [-1, 2]])) )
        cases.append( ((4,),matrix([[ 2,-1, 0, 0],
                                    [-1, 2,-1, 0],
                                    [ 0,-1, 2,-1],
                                    [ 0, 0,-1, 2]])) )
        
        # 2D        
        cases.append( ((1,1), matrix([[4]])) )
        cases.append( ((2,1), matrix([[ 4,-1],
                                      [-1, 4]])) )
        cases.append( ((1,2), matrix([[ 4,-1],
                                      [-1, 4]])) )
        cases.append( ((1,3), matrix([[ 4,-1, 0],
                                      [-1, 4,-1],
                                      [ 0,-1, 4]])) )
        cases.append( ((2,2), matrix([[ 4,-1,-1, 0],
                                      [-1, 4, 0,-1],
                                      [-1, 0, 4,-1],
                                      [ 0,-1,-1, 4]])) )
        # 3D
        cases.append( ((2,2,1), matrix([[ 6,-1,-1, 0],
                                        [-1, 6, 0,-1],
                                        [-1, 0, 6,-1],
                                        [ 0,-1,-1, 6]])) )
        cases.append( ((2,2,2), matrix([[ 6,-1,-1, 0,-1, 0, 0, 0],
                                        [-1, 6, 0,-1, 0,-1, 0, 0],
                                        [-1, 0, 6,-1, 0, 0,-1, 0],
                                        [ 0,-1,-1, 6, 0, 0, 0,-1],
                                        [-1, 0, 0, 0, 6,-1,-1, 0],
                                        [ 0,-1, 0, 0,-1, 6, 0,-1],
                                        [ 0, 0,-1, 0,-1, 0, 6,-1],
                                        [ 0, 0, 0,-1, 0,-1,-1, 6]])) )

        for grid,expected in cases:
            assert_equal( poisson(grid).todense(), expected )


class TestGaugeLaplacian(TestCase):
    def test_gaugelaplacian(self):
        cases = []
        
        beta = 0.0; npts=5
        A = gauge_laplacian( npts, spacing=1.0, beta=beta)
        cases.append( (A, beta) )
        
        beta = 0.11; npts=3;
        A = gauge_laplacian( npts, spacing=1.0, beta=beta)
        cases.append( (A, beta) )
        npts=8
        A = gauge_laplacian( npts, spacing=1.0, beta=beta)
        cases.append( (A, beta) )
        A = gauge_laplacian( npts, spacing=0.1, beta=beta)
        cases.append( (A, beta) )

        beta = 0.42
        A = gauge_laplacian( npts, spacing=1.0, beta=beta)
        cases.append( (A, beta) )
        A = gauge_laplacian( npts, spacing=11.1, beta=beta)
        cases.append( (A, beta) )

        for A,beta in cases:
            # Check Hermitian
            diff = A - A.H
            assert_equal(diff.data, array([]))

            # Check for definiteness
            e = eigvals(A.todense())
            if beta == 0.0:
                # Here, semi-definiteness
                assert_almost_equal(min(abs(e)), 0.0)
            else:
                # zero imaginary part
                assert_almost_equal( min(abs(imag(e))), 0.0)
                # positive real part
                assert( min(real(e)) > 0.0 )




"""Classical AMG"""

from classical import *
from split import *
from interpolate import *
from cr import *

__all__ = [s for s in dir() if not s.startswith('_')]
from pyamg.testing import Tester
test = Tester().test



#!/usr/bin/env python
"""PyAMG: Algebraic Multigrid Solvers in Python

PyAMG is a library of Algebraic Multigrid (AMG) solvers
with a convenient Python interface.

"""

import os
import sys
import subprocess

if sys.version_info[:2] < (2, 6) or (3, 0) <= sys.version_info[0:2]:
    raise RuntimeError("Python version 2.6, 2.7 required.")

if sys.version_info[0] >= 3:
    import builtins
else:
    import __builtin__ as builtins

DOCLINES = __doc__.split('\n')

CLASSIFIERS = """\
Development Status :: 5 - Production/Stable
Environment :: Console
Environment :: X11 Applications
Intended Audience :: Developers
Intended Audience :: Education
Intended Audience :: Science/Research
License :: OSI Approved :: BSD License
Natural Language :: English
Operating System :: OS Independent
Programming Language :: C++
Programming Language :: Python
Topic :: Education
Topic :: Scientific/Engineering
Topic :: Scientific/Engineering :: Mathematics
Topic :: Software Development :: Libraries :: Python Modules
"""

NAME = 'pyamg'
AUTHOR = 'Nathan Bell, Luke OLson, and Jacob Schroder'
AUTHOR_EMAIL = 'luke.olson@gmail.com'
MAINTAINER = 'Luke Olson'
MAINTAINER_EMAIL = 'luke.olson@gmail.com'
URL = 'https://github.com/pyamg/pyamg'
DOWNLOAD_URL = 'https://github.com/pyamg/pyamg/releases'
DESCRIPTION = DOCLINES[0]
LONG_DESCRIPTION = '\n'.join(DOCLINES[2:])
CLASSIFIERS = [_f for _f in CLASSIFIERS.split('\n') if _f]
PLATFORMS = ['Windows', 'Linux', 'Solaris', 'Mac OS-X', 'Unix']
LICENSE = 'BSD'
MAJOR = 2
MINOR = 2
MICRO = 0
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)

# Use NumPy/SciPy style versioning
#
# This changes the pyamg/version.py file directly
#
#       notes for the future:
#       should we try to use a tag if it exists? probably not
#       could also add -devN:C:D:R to the versioning
#       N: revisions numbers approximated with git rev-list HEAD --count
#       C: unique SHA1 generated with git describe --always
#       D: date in format YYYYMMDDHHMMSS
#       R: which repository (fork) this came from with git remote show origin
#       then add this to say fullversion.py


# Return the git revision as a string
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                               env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'describe', '--always'])
        GIT_REVISION = out.strip()
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION

# BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
# update it when the contents of directories change.
if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

# This is a bit hackish: we are setting a global variable so that the main
# pyamg __init__ can detect if it is being loaded by the setup routine, to
# avoid attempting to load components that aren't built yet.  While ugly, it's
# a lot more robust than what was previously being used.
builtins.__PYAMG_SETUP__ = True


def write_version_py(filename='pyamg/version.py'):
    cnt = """
# this file is generated from pyamg in setup.py
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s

if not release:
    version = full_version
"""
    # Adding the git rev number needs to be done inside write_version_py(),
    # otherwise the import of pyamg.version messes up the build under Python 3.
    FULLVERSION = VERSION
    if os.path.exists('.git'):
        GIT_REVISION = git_version()
    elif os.path.exists('pyamg/version.py'):
        # must be a source distribution, use existing version file
        try:
            from pyamg.version import git_revision as GIT_REVISION
        except ImportError:
            raise ImportError("Unable to import git_revision. Try removing "
                              "pyamg/version.py and the build directory "
                              "before building.")
    else:
        GIT_REVISION = "Unknown"

    if not ISRELEASED:
        FULLVERSION += '.dev-' + GIT_REVISION[:7]

    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION,
                       'full_version': FULLVERSION,
                       'git_revision': GIT_REVISION,
                       'isrelease': str(ISRELEASED)})
    finally:
        a.close()


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration(None, parent_package, top_path)
    config.set_options(ignore_setup_xxx_py=True,
                       assume_default_configuration=True,
                       delegate_options_to_subpackages=True,
                       quiet=True)

    config.add_subpackage('pyamg')

    config.get_version('pyamg/version.py')  # sets config.version

    return config


def setup_package():

    old_path = os.getcwd()
    local_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    os.chdir(local_path)
    sys.path.insert(0, local_path)

    # rewrite the version file
    write_version_py()

    # Run build
    from numpy.distutils.core import setup

    try:
        setup(
            name=NAME,
            maintainer=MAINTAINER,
            maintainer_email=MAINTAINER_EMAIL,
            description=DOCLINES[0],
            long_description=LONG_DESCRIPTION,
            url=URL,
            download_url=DOWNLOAD_URL,
            license=LICENSE,
            classifiers=CLASSIFIERS,
            author=AUTHOR,
            author_email=AUTHOR_EMAIL,
            platforms=PLATFORMS,
            configuration=configuration)
    finally:
        del sys.path[0]
        os.chdir(old_path)

    return

if __name__ == '__main__':
    setup_package()



from pyamg.testing import *
from pyamg.krylov import *
from pyamg.krylov._gmres_householder import gmres_householder
from pyamg.krylov._gmres_mgs import gmres_mgs
from numpy import array, zeros, ones
from scipy import mat, random
from scipy.linalg import solve
from pyamg.util.linalg import norm
import pyamg


class TestKrylov(TestCase):
    def setUp(self):
        self.cases=[]
        self.spd_cases=[]
        self.symm_cases=[]
        
        #self.oblique = [gmres, fgmres, cgnr, krylov._gmres_householder.gmres_householder, krylov._gmres_mgs.gmres_mgs]
        self.oblique = [gmres_householder, gmres_mgs, gmres, fgmres, cgnr]
        self.symm_oblique = [cr]
        self.orth = [cgne]
        self.inexact = [bicgstab]
        self.spd_orth = [cg]
        
        # 1x1 
        A = mat([[  1.2]])
        b = array([3.9]).reshape(-1,1)
        x0 = zeros((1,1))
        self.cases.append({'A' : A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 1, 'reduction_factor' : 1e-10})
        self.spd_cases.append({'A' : A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 1, 'reduction_factor' : 1e-10})
        self.symm_cases.append({'A' : A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 1, 'reduction_factor' : 1e-10})


        # 4x4 
        A = mat([[  1.2,    0.,   0.,     0.],
                 [  0.,     4.,   2.,     6.],
                 [  0.,     0.,   9.3,  -2.31],
                 [ -4.,     0.,   0.,    -11.]])
        b = array([1., 3.9, 0., -1.23]).reshape(-1,1)
        x0 = zeros((4,1))
        self.cases.append({'A' : A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 4, 'reduction_factor' : 1e-10})
        self.spd_cases.append({'A' : A.T*A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 4, 'reduction_factor' : 1e-10})
        self.symm_cases.append({'A' : A.T + A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 4, 'reduction_factor' : 1e-10})
        
        # 4x4 Imaginary
        A = mat(A, dtype=complex)
        A[0,0] += 3.1j
        A[3,3] -= 1.34j
        A[1,3] *= 1.0j
        A[1,2] += 1.0j
        b = array([1. - 1.0j, 2.0 - 3.9j, 0., -1.23]).reshape(-1,1)
        x0 = ones((4,1))
        self.cases.append({'A' : A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 4, 'reduction_factor' : 1e-10})
        self.spd_cases.append({'A' : A.H*A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 4, 'reduction_factor' : 1e-10})
        self.symm_cases.append({'A' : A.H + A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 4, 'reduction_factor' : 1e-10})

        # 10x10 
        A = mat([[-1.1,    0.,   0.,   0.,  3.9,  0.,   0.,  11.,  -1.,  0.],
                 [  0.,    4.,   2.9,  0.,   0.,  6.8,  0.,  0.,    0.,  0.],
                 [  0.,    0.,   9.0,  0.,   0.,  0.8,  1., -2.2,   0.,  9.],
                 [ -4.,    0.,   0.0,  0.,   0.,  0.0,  2.,  2.2,   0.,  0.],
                 [  0.,    0.,   0.0, 21.,   0.,  0.1,  0.,   0.,   0.,  0.],
                 [  0.,    0.,   0.0,  0.,  -4.7, 0.0,  0.,   0.,   0.,  0.],
                 [  2.1,   7.,  22.0,  0.,   0.,  0.0,  0.,   0.,   0.,  0.],
                 [  0.,    0.,   0.0, 34.,   0.,  0.0,  0.,   0.,  -12.3,0.],
                 [  0.,   3.4,   0.0,  0.,   0., -0.3,  0.,   0.,   0.,  0.],
                 [  9.,    0.,   0.0,  0.,  87.,  0.0,  0.,   0.,   0.,-11.2]])
        b = array([1., 0., 0.2, 8., 0., -1.9, 11.3, 0.0, 0.1, 0.0]).reshape(-1,1)
        x0 = zeros((10,1))
        x0[4] = 11.1; x0[7] = -2.1
        self.cases.append({'A' : A, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 2, 'reduction_factor' : 0.98})
        self.symm_cases.append({'A' : A + A.T, 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 2, 'reduction_factor' : 0.98})
        self.spd_cases.append({'A' : mat(pyamg.gallery.poisson((10,)).todense()), 'b' : b, 'x0' : x0, 'tol' : 1e-16, 'maxiter' : 2, 'reduction_factor' : 0.98})

    from numpy.testing import dec
    @dec.knownfailureif(True, "This test is known to fail because of complex matrices and Housholder.")
    def test_gmres(self):
        # Ensure repeatability
        random.seed(0)
        
        #  For these small matrices, Householder and MGS GMRES should give the same result, 
        #  and for symmetric (but possibly indefinite) matrices CR and GMRES should give same result
        for maxiter in [1,2,3]:
            for case, symm_case in zip(self.cases, self.symm_cases):
                A = case['A'] 
                b = case['b']
                x0 = case['x0']
                A_symm = symm_case['A'] 
                b_symm = symm_case['b']
                x0_symm = symm_case['x0']
                
                # Test agreement between Householder and GMRES
                (x, flag) = gmres_householder(A,b,x0=x0,maxiter=min(A.shape[0],maxiter))
                (x2, flag2) = gmres_mgs(A,b,x0=x0,maxiter=min(A.shape[0],maxiter))
                assert_array_almost_equal(x/norm(x), x2/norm(x2), 
                        err_msg='Householder GMRES and MGS GMRES gave different results for small matrix')
                assert_equal(flag, flag2, 
                        err_msg='Householder GMRES and MGS GMRES returned different convergence flags for small matrix')

                # Test agreement between GMRES and CR
                if A_symm.shape[0] > 1:
                    residuals2 = []
                    (x2, flag2) = gmres_mgs(A_symm, b_symm, x0=x0_symm, maxiter=min(A.shape[0],maxiter),residuals=residuals2)
                    residuals3 = []
                    (x3, flag2) = cr(A_symm, b_symm, x0=x0_symm, maxiter=min(A.shape[0],maxiter),residuals=residuals3)
                    residuals2 = array(residuals2)
                    residuals3 = array(residuals3)
                    assert_array_almost_equal(residuals3/norm(residuals3), residuals2/norm(residuals2), 
                            err_msg='CR and GMRES yield different residual vectors')
                    assert_array_almost_equal(x2/norm(x2), x3/norm(x3), err_msg='CR and GMRES yield different answers')
            
    from numpy.testing import dec
    @dec.knownfailureif(True, "This test is known to fail because of complex matrices and Housholder.")
    def test_krylov(self):
        # Oblique projectors reduce the residual
        for method in self.oblique:
            for case in self.cases:
                A = case['A']; b = case['b']; x0 = case['x0']
                (xNew, flag) = method(A, b, x0=x0, tol=case['tol'], maxiter=case['maxiter'])
                xNew = xNew.reshape(-1,1)
                assert_equal( (norm(b - A*xNew)/norm(b - A*x0)) < case['reduction_factor'], True, err_msg='Oblique Krylov Method Failed Test')
        
        # Oblique projectors reduce the residual, here we consider oblique projectors for symmetric matrices
        for method in self.symm_oblique:
            for case in self.symm_cases:
                A = case['A']; b = case['b']; x0 = case['x0']
                (xNew, flag) = method(A, b, x0=x0, tol=case['tol'], maxiter=case['maxiter'])
                xNew = xNew.reshape(-1,1)
                assert_equal( (norm(b - A*xNew)/norm(b - A*x0)) < case['reduction_factor'], True, err_msg='Symmetric oblique Krylov Method Failed Test')
    
        # Orthogonal projectors reduce the error
        for method in self.orth:
            for case in self.cases:
                A = case['A']; b = case['b']; x0 = case['x0']
                (xNew, flag) = method(A, b, x0=x0, tol=case['tol'], maxiter=case['maxiter'])
                xNew = xNew.reshape(-1,1)
                soln = solve(A,b)
                assert_equal( (norm(soln - xNew)/norm(soln - x0)) < case['reduction_factor'], True, err_msg='Orthogonal Krylov Method Failed Test')
    
        # SPD Orthogonal projectors reduce the error
        for method in self.spd_orth:
            for case in self.spd_cases:
                A = case['A']; b = case['b']; x0 = case['x0']
                (xNew, flag) = method(A, b, x0=x0, tol=case['tol'], maxiter=case['maxiter'])
                xNew = xNew.reshape(-1,1)
                soln = solve(A,b)
                assert_equal( (norm(soln - xNew)/norm(soln - x0)) < case['reduction_factor'], True, err_msg='Orthogonal Krylov Method Failed Test')

        # Assume that Inexact Methods reduce the residual for these examples
        for method in self.inexact:
            for case in self.cases:
                A = case['A']; b = case['b']; x0 = case['x0']
                (xNew, flag) = method(A, b, x0=x0, tol=case['tol'], maxiter=A.shape[0])
                xNew = xNew.reshape(-1,1)
                assert_equal( (norm(b - A*xNew)/norm(b - A*x0)) < 0.15, True, err_msg='Inexact Krylov Method Failed Test')




""" Test 1, 2, 8 element meshes writing the vtu file.  Validate as well-formed
xml."""

from pyamg.testing import *

import tempfile

import xml.parsers.expat
from numpy import array, uint32

from pyamg.vis import write_vtu

class TestWriteVtu(TestCase):
    def setUp(self):
        cases = []
        class mesh:
            file_name = tempfile.mktemp()
            Verts  = None
            E2V   = None
            pdata = None
            cdata = None
        mesh=mesh()

        # 1 triangle
        mesh.Verts = array([[0.0,0.0],
                           [0.0,1.0],
                           [1.0,1.0]])
        E2V = array([[0,2,1]],uint32)
        mesh.Cells = {5:E2V}
        mesh.pdata = None
        mesh.cdata = None
        cases.append(mesh)
        
        # 2 triangles
        mesh.Verts = array([[0.0,0.0],
                           [1.0,0.0],
                           [0.0,1.0],
                           [1.0,1.0]])
        E2V = array([[0,3,2],
                     [0,1,3]],uint32)
        mesh.Cells = {5:E2V}
        mesh.pdata = None
        mesh.cdata = None
        cases.append(mesh)

        # 8 triangles
        mesh.Verts = array([[0.0,0.0],
                           [1.0,0.0],
                           [2.0,0.0],
                           [0.0,1.0],
                           [1.0,1.0],
                           [2.0,1.0],
                           [0.0,2.0],
                           [1.0,2.0],
                           [2.0,2.0]])
        E2V = array([[0,4,3],
                     [0,1,4],
                     [1,5,4],
                     [1,2,5],
                     [3,7,6],
                     [3,4,7],
                     [4,8,7],
                     [4,5,8]],uint32)
        mesh.Cells = {5:E2V}
        mesh.pdata = None
        mesh.cdata = None
        cases.append(mesh)

        self.cases=cases

    def test_xml(self):
        for mesh in self.cases:
            write_vtu(Verts=mesh.Verts, Cells=mesh.Cells, \
                    pdata=mesh.pdata, cdata=mesh.cdata, fname=mesh.file_name)

            try:
                parser = xml.parsers.expat.ParserCreate()
                parser.ParseFile(open(mesh.file_name, 'r'))
            except Exception, ex:
                assert False, 'problem: %s' % (ex)



from pyamg.testing import *

from pyamg.gallery    import poisson
from pyamg import smoothed_aggregation_solver
from pyamg.util.utils import profile_solver
from pyamg.relaxation.smoothing import change_smoothers

methods = ['gauss_seidel',
           'jacobi',
           'richardson',
           'sor',
           'chebyshev',
           'gauss_seidel_ne',
           'jacobi_ne',
           'gauss_seidel_nr',
           'schwarz',
           'strength_based_schwarz']

methods2 = [('gauss_seidel', 'richardson'),
            ('gauss_seidel', 'jacobi'),
            ('chebyshev', 'sor'),
            (['gauss_seidel', 'chebyshev'], ['sor', 'jacobi']),
            ('gauss_seidel_ne', 'jacobi_ne'),
            (['gauss_seidel_ne', 'gauss_seidel_nr'], 'jacobi_ne'),
            ('cgnr', 'cgne'),
            ('schwarz', 'strength_based_schwarz'),
            ( ('gauss_seidel', {'iterations' : 3}), None),
            ( [('gauss_seidel_ne', {'iterations' : 2}), ('gmres', {'maxiter' : 3})], None),
            ( None, ['cg', 'cgnr', 'cgne']) ]
    
class TestSmoothing(TestCase):
    
    def test_solver_parameters(self):
        A = poisson((50,50), format='csr')

        for method in methods:
            #method = ('richardson', {'omega':4.0/3.0})
            ml = smoothed_aggregation_solver(A, presmoother=method, postsmoother=method, max_coarse=10)

            residuals = profile_solver(ml)
            #print "method",method
            #print "residuals",residuals
            #print "convergence rate:",(residuals[-1]/residuals[0])**(1.0/len(residuals))
            assert( (residuals[-1]/residuals[0])**(1.0/len(residuals)) < 0.95 )

        for method in methods2:
            ml = smoothed_aggregation_solver(A, max_coarse=10)
            change_smoothers(ml, presmoother=method[0], postsmoother=method[1])

            residuals = profile_solver(ml)
            #print "method",method
            #print "residuals",residuals
            #print "convergence rate:",(residuals[-1]/residuals[0])**(1.0/len(residuals))
            assert( (residuals[-1]/residuals[0])**(1.0/len(residuals)) < 0.95 )




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.40
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.
# This file is compatible with both classic and new-style classes.

from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_amg_core', [dirname(__file__)])
        except ImportError:
            import _amg_core
            return _amg_core
        if fp is not None:
            try:
                _mod = imp.load_module('_amg_core', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _amg_core = swig_import_helper()
    del swig_import_helper
else:
    import _amg_core
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static) or hasattr(self,name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0


F_NODE = _amg_core.F_NODE
C_NODE = _amg_core.C_NODE
U_NODE = _amg_core.U_NODE

def cljp_naive_splitting(*args):
  """
    cljp_naive_splitting(int n, int Sp, int Sj, int Tp, int Tj, int splitting, 
        int colorflag)
    """
  return _amg_core.cljp_naive_splitting(*args)

def naive_aggregation(*args):
  """naive_aggregation(int n_row, int Ap, int Aj, int x, int y) -> int"""
  return _amg_core.naive_aggregation(*args)

def standard_aggregation(*args):
  """standard_aggregation(int n_row, int Ap, int Aj, int x, int y) -> int"""
  return _amg_core.standard_aggregation(*args)

def rs_cf_splitting(*args):
  """rs_cf_splitting(int n_nodes, int Sp, int Sj, int Tp, int Tj, int splitting)"""
  return _amg_core.rs_cf_splitting(*args)

def rs_direct_interpolation_pass1(*args):
  """rs_direct_interpolation_pass1(int n_nodes, int Sp, int Sj, int splitting, int Bp)"""
  return _amg_core.rs_direct_interpolation_pass1(*args)

def maximal_independent_set_serial(*args):
  """
    maximal_independent_set_serial(int num_rows, int Ap, int Aj, int active, int C, int F, 
        int x) -> int
    """
  return _amg_core.maximal_independent_set_serial(*args)

def vertex_coloring_mis(*args):
  """vertex_coloring_mis(int num_rows, int Ap, int Aj, int x) -> int"""
  return _amg_core.vertex_coloring_mis(*args)

def vertex_coloring_jones_plassmann(*args):
  """vertex_coloring_jones_plassmann(int num_rows, int Ap, int Aj, int x, double y) -> int"""
  return _amg_core.vertex_coloring_jones_plassmann(*args)

def vertex_coloring_LDF(*args):
  """vertex_coloring_LDF(int num_rows, int Ap, int Aj, int x, double y) -> int"""
  return _amg_core.vertex_coloring_LDF(*args)

def breadth_first_search(*args):
  """breadth_first_search(int Ap, int Aj, int seed, int order, int level)"""
  return _amg_core.breadth_first_search(*args)

def connected_components(*args):
  """connected_components(int num_nodes, int Ap, int Aj, int components) -> int"""
  return _amg_core.connected_components(*args)


def signof(*args):
  """
    signof(int a) -> int
    signof(float a) -> float
    signof(double a) -> double
    """
  return _amg_core.signof(*args)

def conjugate(*args):
  """
    conjugate(float x) -> float
    conjugate(double x) -> double
    conjugate(npy_cfloat_wrapper x) -> npy_cfloat_wrapper
    conjugate(npy_cdouble_wrapper x) -> npy_cdouble_wrapper
    """
  return _amg_core.conjugate(*args)

def real(*args):
  """
    real(float x) -> float
    real(double x) -> double
    real(npy_cfloat_wrapper x) -> float
    real(npy_cdouble_wrapper x) -> double
    """
  return _amg_core.real(*args)

def imag(*args):
  """
    imag(float x) -> float
    imag(double x) -> double
    imag(npy_cfloat_wrapper x) -> float
    imag(npy_cdouble_wrapper x) -> double
    """
  return _amg_core.imag(*args)

def mynorm(*args):
  """
    mynorm(float x) -> float
    mynorm(double x) -> double
    mynorm(npy_cfloat_wrapper x) -> float
    mynorm(npy_cdouble_wrapper x) -> double
    """
  return _amg_core.mynorm(*args)

def mynormsq(*args):
  """
    mynormsq(float x) -> float
    mynormsq(double x) -> double
    mynormsq(npy_cfloat_wrapper x) -> float
    mynormsq(npy_cdouble_wrapper x) -> double
    """
  return _amg_core.mynormsq(*args)

def zero_real(*args):
  """
    zero_real(float x) -> float
    zero_real(double x) -> double
    zero_real(npy_cfloat_wrapper x) -> npy_cfloat_wrapper
    zero_real(npy_cdouble_wrapper x) -> npy_cdouble_wrapper
    """
  return _amg_core.zero_real(*args)

def zero_imag(*args):
  """
    zero_imag(float x) -> float
    zero_imag(double x) -> double
    zero_imag(npy_cfloat_wrapper x) -> npy_cfloat_wrapper
    zero_imag(npy_cdouble_wrapper x) -> npy_cdouble_wrapper
    """
  return _amg_core.zero_imag(*args)

def rs_direct_interpolation_pass2(*args):
  """
    rs_direct_interpolation_pass2(int n_nodes, int Ap, int Aj, float Ax, int Sp, int Sj, 
        float Sx, int splitting, int Bp, int Bj, 
        float Bx)
    rs_direct_interpolation_pass2(int n_nodes, int Ap, int Aj, double Ax, int Sp, int Sj, 
        double Sx, int splitting, int Bp, int Bj, 
        double Bx)
    """
  return _amg_core.rs_direct_interpolation_pass2(*args)

def satisfy_constraints_helper(*args):
  """
    satisfy_constraints_helper(int RowsPerBlock, int ColsPerBlock, int num_block_rows, 
        int NullDim, float x, float y, float z, 
        int Sp, int Sj, float Sx)
    satisfy_constraints_helper(int RowsPerBlock, int ColsPerBlock, int num_block_rows, 
        int NullDim, double x, double y, double z, 
        int Sp, int Sj, double Sx)
    satisfy_constraints_helper(int RowsPerBlock, int ColsPerBlock, int num_block_rows, 
        int NullDim, npy_cfloat_wrapper x, npy_cfloat_wrapper y, 
        npy_cfloat_wrapper z, int Sp, 
        int Sj, npy_cfloat_wrapper Sx)
    satisfy_constraints_helper(int RowsPerBlock, int ColsPerBlock, int num_block_rows, 
        int NullDim, npy_cdouble_wrapper x, npy_cdouble_wrapper y, 
        npy_cdouble_wrapper z, int Sp, 
        int Sj, npy_cdouble_wrapper Sx)
    """
  return _amg_core.satisfy_constraints_helper(*args)

def calc_BtB(*args):
  """
    calc_BtB(int NullDim, int Nnodes, int ColsPerBlock, float b, 
        int BsqCols, float x, int Sp, int Sj)
    calc_BtB(int NullDim, int Nnodes, int ColsPerBlock, double b, 
        int BsqCols, double x, int Sp, int Sj)
    calc_BtB(int NullDim, int Nnodes, int ColsPerBlock, npy_cfloat_wrapper b, 
        int BsqCols, npy_cfloat_wrapper x, 
        int Sp, int Sj)
    calc_BtB(int NullDim, int Nnodes, int ColsPerBlock, npy_cdouble_wrapper b, 
        int BsqCols, npy_cdouble_wrapper x, 
        int Sp, int Sj)
    """
  return _amg_core.calc_BtB(*args)

def incomplete_mat_mult_bsr(*args):
  """
    incomplete_mat_mult_bsr(int Ap, int Aj, float Ax, int Bp, int Bj, float Bx, 
        int Sp, int Sj, float Sx, int n_brow, int n_bcol, 
        int brow_A, int bcol_A, int bcol_B)
    incomplete_mat_mult_bsr(int Ap, int Aj, double Ax, int Bp, int Bj, double Bx, 
        int Sp, int Sj, double Sx, int n_brow, int n_bcol, 
        int brow_A, int bcol_A, int bcol_B)
    incomplete_mat_mult_bsr(int Ap, int Aj, npy_cfloat_wrapper Ax, int Bp, int Bj, 
        npy_cfloat_wrapper Bx, int Sp, int Sj, npy_cfloat_wrapper Sx, 
        int n_brow, int n_bcol, 
        int brow_A, int bcol_A, int bcol_B)
    incomplete_mat_mult_bsr(int Ap, int Aj, npy_cdouble_wrapper Ax, int Bp, int Bj, 
        npy_cdouble_wrapper Bx, int Sp, int Sj, 
        npy_cdouble_wrapper Sx, int n_brow, int n_bcol, 
        int brow_A, int bcol_A, int bcol_B)
    """
  return _amg_core.incomplete_mat_mult_bsr(*args)

def maximum_row_value(*args):
  """
    maximum_row_value(int n_row, float x, int Ap, int Aj, float Ax)
    maximum_row_value(int n_row, double x, int Ap, int Aj, double Ax)
    maximum_row_value(int n_row, npy_cfloat_wrapper x, int Ap, int Aj, npy_cfloat_wrapper Ax)
    maximum_row_value(int n_row, npy_cdouble_wrapper x, int Ap, int Aj, npy_cdouble_wrapper Ax)
    """
  return _amg_core.maximum_row_value(*args)

def pinv_array(*args):
  """
    pinv_array(float Ax, int m, int n, char TransA)
    pinv_array(double Ax, int m, int n, char TransA)
    pinv_array(npy_cfloat_wrapper Ax, int m, int n, char TransA)
    pinv_array(npy_cdouble_wrapper Ax, int m, int n, char TransA)
    """
  return _amg_core.pinv_array(*args)

def classical_strength_of_connection(*args):
  """
    classical_strength_of_connection(int n_row, float theta, int Ap, int Aj, float Ax, int Sp, 
        int Sj, float Sx)
    classical_strength_of_connection(int n_row, double theta, int Ap, int Aj, double Ax, 
        int Sp, int Sj, double Sx)
    classical_strength_of_connection(int n_row, float theta, int Ap, int Aj, npy_cfloat_wrapper Ax, 
        int Sp, int Sj, npy_cfloat_wrapper Sx)
    classical_strength_of_connection(int n_row, double theta, int Ap, int Aj, npy_cdouble_wrapper Ax, 
        int Sp, int Sj, npy_cdouble_wrapper Sx)
    """
  return _amg_core.classical_strength_of_connection(*args)

def symmetric_strength_of_connection(*args):
  """
    symmetric_strength_of_connection(int n_row, float theta, int Ap, int Aj, float Ax, int Sp, 
        int Sj, float Sx)
    symmetric_strength_of_connection(int n_row, double theta, int Ap, int Aj, double Ax, 
        int Sp, int Sj, double Sx)
    symmetric_strength_of_connection(int n_row, float theta, int Ap, int Aj, npy_cfloat_wrapper Ax, 
        int Sp, int Sj, npy_cfloat_wrapper Sx)
    symmetric_strength_of_connection(int n_row, double theta, int Ap, int Aj, npy_cdouble_wrapper Ax, 
        int Sp, int Sj, npy_cdouble_wrapper Sx)
    """
  return _amg_core.symmetric_strength_of_connection(*args)

def evolution_strength_helper(*args):
  """
    evolution_strength_helper(float Sx, int Sp, int Sj, int nrows, float x, float y, 
        float b, int BDBCols, int NullDim, float tol)
    evolution_strength_helper(double Sx, int Sp, int Sj, int nrows, double x, double y, 
        double b, int BDBCols, int NullDim, double tol)
    evolution_strength_helper(npy_cfloat_wrapper Sx, int Sp, int Sj, int nrows, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper y, npy_cfloat_wrapper b, 
        int BDBCols, int NullDim, 
        float tol)
    evolution_strength_helper(npy_cdouble_wrapper Sx, int Sp, int Sj, int nrows, 
        npy_cdouble_wrapper x, npy_cdouble_wrapper y, 
        npy_cdouble_wrapper b, int BDBCols, int NullDim, 
        double tol)
    """
  return _amg_core.evolution_strength_helper(*args)

def incomplete_mat_mult_csr(*args):
  """
    incomplete_mat_mult_csr(int Ap, int Aj, float Ax, int Bp, int Bj, float Bx, 
        int Sp, int Sj, float Sx, int num_rows)
    incomplete_mat_mult_csr(int Ap, int Aj, double Ax, int Bp, int Bj, double Bx, 
        int Sp, int Sj, double Sx, int num_rows)
    incomplete_mat_mult_csr(int Ap, int Aj, npy_cfloat_wrapper Ax, int Bp, int Bj, 
        npy_cfloat_wrapper Bx, int Sp, int Sj, npy_cfloat_wrapper Sx, 
        int num_rows)
    incomplete_mat_mult_csr(int Ap, int Aj, npy_cdouble_wrapper Ax, int Bp, int Bj, 
        npy_cdouble_wrapper Bx, int Sp, int Sj, 
        npy_cdouble_wrapper Sx, int num_rows)
    """
  return _amg_core.incomplete_mat_mult_csr(*args)

def apply_distance_filter(*args):
  """
    apply_distance_filter(int n_row, float epsilon, int Sp, int Sj, float Sx)
    apply_distance_filter(int n_row, double epsilon, int Sp, int Sj, double Sx)
    """
  return _amg_core.apply_distance_filter(*args)

def apply_absolute_distance_filter(*args):
  """
    apply_absolute_distance_filter(int n_row, float epsilon, int Sp, int Sj, float Sx)
    apply_absolute_distance_filter(int n_row, double epsilon, int Sp, int Sj, double Sx)
    """
  return _amg_core.apply_absolute_distance_filter(*args)

def min_blocks(*args):
  """
    min_blocks(int n_blocks, int blocksize, float Sx, float Tx)
    min_blocks(int n_blocks, int blocksize, double Sx, double Tx)
    """
  return _amg_core.min_blocks(*args)

def bsr_gauss_seidel(*args):
  """
    bsr_gauss_seidel(int Ap, int Aj, float Ax, float x, float b, int row_start, 
        int row_stop, int row_step, int blocksize)
    bsr_gauss_seidel(int Ap, int Aj, double Ax, double x, double b, int row_start, 
        int row_stop, int row_step, int blocksize)
    bsr_gauss_seidel(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, int row_start, 
        int row_stop, int row_step, int blocksize)
    bsr_gauss_seidel(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, int row_start, 
        int row_stop, int row_step, int blocksize)
    """
  return _amg_core.bsr_gauss_seidel(*args)

def bsr_jacobi(*args):
  """
    bsr_jacobi(int Ap, int Aj, float Ax, float x, float b, float temp, 
        int row_start, int row_stop, int row_step, 
        int blocksize, float omega)
    bsr_jacobi(int Ap, int Aj, double Ax, double x, double b, double temp, 
        int row_start, int row_stop, int row_step, 
        int blocksize, double omega)
    bsr_jacobi(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, npy_cfloat_wrapper temp, 
        int row_start, int row_stop, int row_step, 
        int blocksize, npy_cfloat_wrapper omega)
    bsr_jacobi(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, npy_cdouble_wrapper temp, 
        int row_start, int row_stop, 
        int row_step, int blocksize, npy_cdouble_wrapper omega)
    """
  return _amg_core.bsr_jacobi(*args)

def gauss_seidel(*args):
  """
    gauss_seidel(int Ap, int Aj, float Ax, float x, float b, int row_start, 
        int row_stop, int row_step)
    gauss_seidel(int Ap, int Aj, double Ax, double x, double b, int row_start, 
        int row_stop, int row_step)
    gauss_seidel(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, int row_start, 
        int row_stop, int row_step)
    gauss_seidel(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, int row_start, 
        int row_stop, int row_step)
    """
  return _amg_core.gauss_seidel(*args)

def jacobi(*args):
  """
    jacobi(int Ap, int Aj, float Ax, float x, float b, float temp, 
        int row_start, int row_stop, int row_step, 
        float omega)
    jacobi(int Ap, int Aj, double Ax, double x, double b, double temp, 
        int row_start, int row_stop, int row_step, 
        double omega)
    jacobi(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, npy_cfloat_wrapper temp, 
        int row_start, int row_stop, int row_step, 
        npy_cfloat_wrapper omega)
    jacobi(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, npy_cdouble_wrapper temp, 
        int row_start, int row_stop, 
        int row_step, npy_cdouble_wrapper omega)
    """
  return _amg_core.jacobi(*args)

def block_jacobi(*args):
  """
    block_jacobi(int Ap, int Aj, float Ax, float x, float b, float Tx, 
        float temp, int row_start, int row_stop, int row_step, 
        float omega, int blocksize)
    block_jacobi(int Ap, int Aj, double Ax, double x, double b, double Tx, 
        double temp, int row_start, int row_stop, 
        int row_step, double omega, int blocksize)
    block_jacobi(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, npy_cfloat_wrapper Tx, 
        npy_cfloat_wrapper temp, int row_start, 
        int row_stop, int row_step, npy_cfloat_wrapper omega, 
        int blocksize)
    block_jacobi(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, npy_cdouble_wrapper Tx, 
        npy_cdouble_wrapper temp, int row_start, 
        int row_stop, int row_step, npy_cdouble_wrapper omega, 
        int blocksize)
    """
  return _amg_core.block_jacobi(*args)

def block_gauss_seidel(*args):
  """
    block_gauss_seidel(int Ap, int Aj, float Ax, float x, float b, float Tx, 
        int row_start, int row_stop, int row_step, 
        int blocksize)
    block_gauss_seidel(int Ap, int Aj, double Ax, double x, double b, double Tx, 
        int row_start, int row_stop, int row_step, 
        int blocksize)
    block_gauss_seidel(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, npy_cfloat_wrapper Tx, 
        int row_start, int row_stop, int row_step, 
        int blocksize)
    block_gauss_seidel(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, npy_cdouble_wrapper Tx, 
        int row_start, int row_stop, int row_step, 
        int blocksize)
    """
  return _amg_core.block_gauss_seidel(*args)

def gauss_seidel_indexed(*args):
  """
    gauss_seidel_indexed(int Ap, int Aj, float Ax, float x, float b, int Id, 
        int row_start, int row_stop, int row_step)
    gauss_seidel_indexed(int Ap, int Aj, double Ax, double x, double b, int Id, 
        int row_start, int row_stop, int row_step)
    gauss_seidel_indexed(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, int Id, int row_start, 
        int row_stop, int row_step)
    gauss_seidel_indexed(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, int Id, int row_start, 
        int row_stop, int row_step)
    """
  return _amg_core.gauss_seidel_indexed(*args)

def jacobi_ne(*args):
  """
    jacobi_ne(int Ap, int Aj, float Ax, float x, float b, float Tx, 
        float temp, int row_start, int row_stop, int row_step, 
        float omega)
    jacobi_ne(int Ap, int Aj, double Ax, double x, double b, double Tx, 
        double temp, int row_start, int row_stop, 
        int row_step, double omega)
    jacobi_ne(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, npy_cfloat_wrapper Tx, 
        npy_cfloat_wrapper temp, int row_start, 
        int row_stop, int row_step, npy_cfloat_wrapper omega)
    jacobi_ne(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, npy_cdouble_wrapper Tx, 
        npy_cdouble_wrapper temp, int row_start, 
        int row_stop, int row_step, npy_cdouble_wrapper omega)
    """
  return _amg_core.jacobi_ne(*args)

def gauss_seidel_ne(*args):
  """
    gauss_seidel_ne(int Ap, int Aj, float Ax, float x, float b, int row_start, 
        int row_stop, int row_step, float Tx, 
        float omega)
    gauss_seidel_ne(int Ap, int Aj, double Ax, double x, double b, int row_start, 
        int row_stop, int row_step, double Tx, 
        double omega)
    gauss_seidel_ne(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, int row_start, 
        int row_stop, int row_step, npy_cfloat_wrapper Tx, 
        float omega)
    gauss_seidel_ne(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, int row_start, 
        int row_stop, int row_step, npy_cdouble_wrapper Tx, 
        double omega)
    """
  return _amg_core.gauss_seidel_ne(*args)

def gauss_seidel_nr(*args):
  """
    gauss_seidel_nr(int Ap, int Aj, float Ax, float x, float z, int col_start, 
        int col_stop, int col_step, float Tx, 
        float omega)
    gauss_seidel_nr(int Ap, int Aj, double Ax, double x, double z, int col_start, 
        int col_stop, int col_step, double Tx, 
        double omega)
    gauss_seidel_nr(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper z, int col_start, 
        int col_stop, int col_step, npy_cfloat_wrapper Tx, 
        float omega)
    gauss_seidel_nr(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper z, int col_start, 
        int col_stop, int col_step, npy_cdouble_wrapper Tx, 
        double omega)
    """
  return _amg_core.gauss_seidel_nr(*args)

def overlapping_schwarz_csr(*args):
  """
    overlapping_schwarz_csr(int Ap, int Aj, float Ax, float x, float b, float Tx, 
        int Tp, int Sj, int Sp, int nsdomains, int nrows, 
        int row_start, int row_stop, int row_step)
    overlapping_schwarz_csr(int Ap, int Aj, double Ax, double x, double b, double Tx, 
        int Tp, int Sj, int Sp, int nsdomains, 
        int nrows, int row_start, int row_stop, int row_step)
    overlapping_schwarz_csr(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper x, 
        npy_cfloat_wrapper b, npy_cfloat_wrapper Tx, 
        int Tp, int Sj, int Sp, int nsdomains, 
        int nrows, int row_start, int row_stop, int row_step)
    overlapping_schwarz_csr(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper x, 
        npy_cdouble_wrapper b, npy_cdouble_wrapper Tx, 
        int Tp, int Sj, int Sp, int nsdomains, 
        int nrows, int row_start, int row_stop, 
        int row_step)
    """
  return _amg_core.overlapping_schwarz_csr(*args)

def extract_subblocks(*args):
  """
    extract_subblocks(int Ap, int Aj, float Ax, float Tx, int Tp, int Sj, 
        int Sp, int nsdomains, int nrows)
    extract_subblocks(int Ap, int Aj, double Ax, double Tx, int Tp, int Sj, 
        int Sp, int nsdomains, int nrows)
    extract_subblocks(int Ap, int Aj, npy_cfloat_wrapper Ax, npy_cfloat_wrapper Tx, 
        int Tp, int Sj, int Sp, int nsdomains, 
        int nrows)
    extract_subblocks(int Ap, int Aj, npy_cdouble_wrapper Ax, npy_cdouble_wrapper Tx, 
        int Tp, int Sj, int Sp, int nsdomains, 
        int nrows)
    """
  return _amg_core.extract_subblocks(*args)

def apply_householders(*args):
  """
    apply_householders(float z, float B, int n, int start, int stop, int step)
    apply_householders(double z, double B, int n, int start, int stop, int step)
    apply_householders(npy_cfloat_wrapper z, npy_cfloat_wrapper B, int n, 
        int start, int stop, int step)
    apply_householders(npy_cdouble_wrapper z, npy_cdouble_wrapper B, int n, 
        int start, int stop, int step)
    """
  return _amg_core.apply_householders(*args)

def householder_hornerscheme(*args):
  """
    householder_hornerscheme(float z, float B, float y, int n, int start, int stop, 
        int step)
    householder_hornerscheme(double z, double B, double y, int n, int start, int stop, 
        int step)
    householder_hornerscheme(npy_cfloat_wrapper z, npy_cfloat_wrapper B, npy_cfloat_wrapper y, 
        int n, int start, int stop, int step)
    householder_hornerscheme(npy_cdouble_wrapper z, npy_cdouble_wrapper B, npy_cdouble_wrapper y, 
        int n, int start, int stop, 
        int step)
    """
  return _amg_core.householder_hornerscheme(*args)

def apply_givens(*args):
  """
    apply_givens(float B, float x, int n, int nrot)
    apply_givens(double B, double x, int n, int nrot)
    apply_givens(npy_cfloat_wrapper B, npy_cfloat_wrapper x, int n, 
        int nrot)
    apply_givens(npy_cdouble_wrapper B, npy_cdouble_wrapper x, int n, 
        int nrot)
    """
  return _amg_core.apply_givens(*args)

def maximal_independent_set_parallel(*args):
  """
    maximal_independent_set_parallel(int num_rows, int Ap, int Aj, int active, int C, int F, 
        int x, double y, int max_iters = -1) -> int
    maximal_independent_set_parallel(int num_rows, int Ap, int Aj, int active, int C, int F, 
        int x, double y) -> int
    """
  return _amg_core.maximal_independent_set_parallel(*args)

def maximal_independent_set_k_parallel(*args):
  """
    maximal_independent_set_k_parallel(int num_rows, int Ap, int Aj, int k, int x, double y, 
        int max_iters = -1)
    maximal_independent_set_k_parallel(int num_rows, int Ap, int Aj, int k, int x, double y)
    """
  return _amg_core.maximal_independent_set_k_parallel(*args)

def bellman_ford(*args):
  """
    bellman_ford(int num_rows, int Ap, int Aj, int Ax, int x, int y)
    bellman_ford(int num_rows, int Ap, int Aj, float Ax, float x, int y)
    bellman_ford(int num_rows, int Ap, int Aj, double Ax, double x, 
        int y)
    """
  return _amg_core.bellman_ford(*args)

def lloyd_cluster(*args):
  """
    lloyd_cluster(int num_rows, int Ap, int Aj, int Ax, int num_seeds, 
        int x, int y, int z)
    lloyd_cluster(int num_rows, int Ap, int Aj, float Ax, int num_seeds, 
        float x, int y, int z)
    lloyd_cluster(int num_rows, int Ap, int Aj, double Ax, int num_seeds, 
        double x, int y, int z)
    """
  return _amg_core.lloyd_cluster(*args)

def fit_candidates(*args):
  """
    fit_candidates(int n_row, int n_col, int K1, int K2, int Ap, int Ai, 
        float Ax, float B, float R, float tol)
    fit_candidates(int n_row, int n_col, int K1, int K2, int Ap, int Ai, 
        double Ax, double B, double R, double tol)
    fit_candidates(int n_row, int n_col, int K1, int K2, int Ap, int Ai, 
        npy_cfloat_wrapper Ax, npy_cfloat_wrapper B, 
        npy_cfloat_wrapper R, float tol)
    fit_candidates(int n_row, int n_col, int K1, int K2, int Ap, int Ai, 
        npy_cdouble_wrapper Ax, npy_cdouble_wrapper B, 
        npy_cdouble_wrapper R, double tol)
    """
  return _amg_core.fit_candidates(*args)

